package gus05.entity.gus.app.find.findfile;


import java.io.File;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.tree.TreeNode;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Transform;
import gus05.framework.resources.FileProvider;



public class FindFile implements Entity, Transform {


    public String getName()				{return "gus.app.find.findfile";}
    public String getCreationDate()		{return "2006.10.29";}


    private Pattern tag_pattern = Pattern.compile("<([^>]+)>");
    

    private FileProvider fileProvider;
    private Service resourceCall;
    
    public FindFile() throws Exception
    {
        fileProvider = (FileProvider) Outside.resource(this,"app.files");
        resourceCall = Outside.service(this,"app.entity.resourcecall");
    }



    public Object transform(Object obj) throws Exception
    {
        if(obj instanceof File) return obj;
        if(obj instanceof String)return retrieveFileFromString((String)obj);
        if(obj instanceof TreeNode)return retrieveFileFromString(obj.toString());
        
        throw new Exception("Invalid type: "+obj.getClass().getName());
    }
    
    
    
    
    private File retrieveFileFromString(String text) throws Exception
    {
        text = text.trim();
        if(text.startsWith("path.")) return fileProvider.getFile(text);
        
        try{return (File) resourceCall.transform(text);}
        catch(Exception e){}
        
        return retrieveFileFromPath(text);
    }
    
    
    
    
    private File retrieveFileFromPath(String path) throws Exception
    {
        while(path.startsWith("<"))
        {
            Matcher m = tag_pattern.matcher(path);
            if(!m.find()) throw new Exception("wrong syntax for path: "+path);
            String tag = m.group(1);
            
            File f = fileProvider.getFile(tag);
            if(f==null) throw new Exception("Unknown tag "+tag);
            path = path.replace(m.group(), f.getAbsolutePath());
        }
        path = path.replace("\\",File.separator).replace("/",File.separator);
        return new File(path);
    }
    
    


}

