package gus05.entity.gus.app.watchrelease.menu;


import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Give;
import gus05.framework.resources.IconProvider;
import gus05.framework.resources.StringProvider;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.Icon;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;



public class WatchReleaseMenu implements Entity, Give, Runnable, ActionListener {


    public String getName()				{return "gus.app.watchrelease.menu";}
    public String getCreationDate()		{return "2009.02.15";}

    
    public static final String MENU_newRelease = "MENU_newRelease";
    public static final String MENUITEM_downloadInstall = "MENUITEM_downloadInstall";

    
    
    private Service downloadFile;
    private Service installFile;
    private Service lingSupport;
    
    private StringProvider lingProvider;
    
    
    private JMenu menu;
    private JMenuItem menuItem;
    private JMenuBar menuBar;
    
    private IconProvider iconProvider;
    
    private String timeStamp = "";
    private String fileName = "";
    
    
    private Icon icon(String key)
    {return iconProvider.getIcon(key);}
    
    private String ling(String key)
    {return lingProvider.getString(key);}
    
    
    

    public WatchReleaseMenu() throws Exception
    {
    	downloadFile = Outside.service(this,"gus.app.watchrelease.downloadfile");
    	installFile = Outside.service(this,"gus.app.watchrelease.installfile");
    	lingSupport = Outside.service(this,"app.supportling");
    	
    	lingProvider = (StringProvider) Outside.resource(this,"app.lingstring");
    	iconProvider = (IconProvider) Outside.resource(this,"app.gui.icons");
    	menuBar = (JMenuBar) Outside.resource(this,"menubar");
    	
    	menuItem = new JMenuItem(ling(MENUITEM_downloadInstall),icon("WEB_installRelease"));
    	menuItem.addActionListener(this);
    	
    	menu = new JMenu(ling(MENU_newRelease));
    	menu.add(menuItem);
    	
    	lingSupport.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {languageChanged();}
		});
    }



    public void give(Object obj) throws Exception
    {
    	String[] infos = (String[]) obj;
    	if(infos.length!=2) throw new Exception("Wrong info number: "+infos.length);
    	
    	timeStamp = infos[0];
    	fileName = infos[1];
    	
    	menuItem.setText(ling(MENUITEM_downloadInstall)+fileName);

    	showMenu();
    }
    
    
    
    
    
    private void showMenu()
    {
    	menuBar.add(menu);
		menuBar.revalidate();
		menuBar.repaint();
    }
    
    
    private void hideMenu()
    {
    	menuBar.remove(menu);
		menuBar.repaint();
    }
    
    
    
    
	public void actionPerformed(ActionEvent e)
	{new Thread(this).start();}

	
	
	
	public void run()
	{
		try
		{
			hideMenu();
			downloadFile.give(new String[]{timeStamp,fileName});
			File file = (File) downloadFile.provide();
			installFile.give(file);
		}
		catch(Exception e)
		{Outside.err(this,"run()",e);}
	}
	
	
	
	
	private void languageChanged()
	{
		menu.setText(ling(MENU_newRelease));
		menuItem.setText(ling(MENUITEM_downloadInstall)+fileName);
	}
}

