package gus05.entity.gus.data.editor.map.string.editortool1;


import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Execute;
import gus05.framework.features.Give;
import gus05.framework.features.Graphic;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;



public class MapStringEditorTool1 implements Entity, Give, Graphic {


    public String getName()				{return "gus.data.editor.map.string.editortool1";}
    public String getCreationDate()		{return "2008.06.13";}



    private Service mapToTable;
    private Service saveData;
    private Service pbeEncrypt;
    private Service pbeDecrypt;
    private Service barCopyPastClear;
    private Service quick;
    private Service undo;
    
    private JPanel panel;
    private JTextArea textArea;
    
    private Map map;
    private Map actions;
    
    
    
    
    public MapStringEditorTool1() throws Exception
    {
    	mapToTable = Outside.service(this,"gus.collection.map.string.tostringtable");
    	saveData = Outside.service(this,"gus.file.savedata.tabtoexcel");
    	pbeEncrypt = Outside.service(this,"gus.security.encryption.pbe1.stringcol.encrypt");
    	pbeDecrypt = Outside.service(this,"gus.security.encryption.pbe1.stringcol.decrypt");
    	barCopyPastClear = Outside.service(this,"gus.graphic.textcomp.build.barcopypasteclear");
    	quick = Outside.service(this,"gus.graphic.textcomp.action.w.quickreplace");
    	undo = Outside.service(this,"gus.graphic.textcomp.action.z.undo");
        
    	actions = new HashMap();
    	textArea = new JTextArea();
    	quick.give(textArea);
    	undo.give(textArea);
    	
    	Vector vec_display = initializeActions_display();
    	JPanel p_display = buildPanel("Display",vec_display);
    	
    	Vector vec_modify = initializeActions_modify();
    	JPanel p_modify = buildPanel("Modify",vec_modify);
    	
    	Vector vec_importExport = initializeActions_importExport();
    	JPanel p_importExport = buildPanel("Import / Export",vec_importExport);
    	
    	
    	
    	JPanel p_top = new JPanel(new GridLayout(1,3));
    	p_top.add(p_display);
    	p_top.add(p_modify);
    	p_top.add(p_importExport);
    	
    	JPanel p_bottom = (JPanel) barCopyPastClear.transform(textArea);
    	
    	panel = new JPanel(new BorderLayout());
    	panel.add(p_top,BorderLayout.NORTH);
    	panel.add(new JScrollPane(textArea),BorderLayout.CENTER);
    	panel.add(p_bottom,BorderLayout.SOUTH);
    }
    
    
    
    public void give(Object obj) throws Exception
    {map = (Map)obj;}
    
    

    public JComponent gui()
    {return panel;}
    
    
    
    
    
    private JPanel buildPanel(String text, Vector vec)
    {
    	JPanel p = new JPanel(new GridLayout(2,1));
    	StringJComboBox combo = new StringJComboBox(vec);
    	JButton0 button = new JButton0(text,combo);
    	
    	p.add(combo);
    	p.add(button);
    	return p;
    }
    
    
    
    
    private class JButton0 extends JButton implements ActionListener
    {
    	private StringJComboBox combo;
    	
    	public JButton0(String text, StringJComboBox combo)
    	{
    		super(text);
    		this.combo = combo;
    		addActionListener(this);
    	}

		public void actionPerformed(ActionEvent e)
		{
			String label = (String) combo.getSelectedItem();
	    	perform(label);
		}
    }
    
    
    
    
    
    private void perform(String label)
    {
    	if(map==null) return;
    	
    	try
    	{
    		Execute action = (Execute) actions.get(label);
    		action.execute();
    	}
    	catch(Exception e)
    	{
    		Exception ee = new Exception("FAILED for "+label+"\n"+e);
    		Outside.err(this,"perform(String)",ee);
    	}
    }
    
    
    
    
    
    
    private Vector initializeActions_display()
    {
    	Vector vec = new Vector();
    	
    	add(vec,"Display key(tab)value",new Execute(){
			public void execute() throws Exception
			{displayKeyTabValue();}
		});
    	add(vec,"Display key=value",new Execute(){
			public void execute() throws Exception
			{displayKeyEqValue();}
		});
		add(vec,"Display keys",new Execute(){
			public void execute() throws Exception
			{displayKeys();}
		});
    	add(vec,"Display values",new Execute(){
			public void execute() throws Exception
			{displayValues();}
		});
		add(vec,"Display value(tab)frequencies",new Execute(){
			public void execute() throws Exception
			{displayKeys();}
		});
		return vec;
    }
    
    
   
    
    
    
    
    private Vector initializeActions_modify()
    {
    	Vector vec = new Vector();
    	
		add(vec,"Empty values",new Execute(){
			public void execute() throws Exception
			{emptyValues();}
		});
		add(vec,"Clear all",new Execute(){
			public void execute() throws Exception
			{clearProperties();}
		});
		add(vec,"Invert key value",new Execute(){
			public void execute() throws Exception
			{invertKeyValue();}
		});
		add(vec,"Encrypt prop",new Execute(){
			public void execute() throws Exception
			{encryptProp();}
		});
		add(vec,"Decrypt prop",new Execute(){
			public void execute() throws Exception
			{decryptProp();}
		});
		return vec;
    }
    
    
    
    
    
    
    
    
    private Vector initializeActions_importExport()
    {
    	Vector vec = new Vector();
    	
		add(vec,"Replace: key(tab)value",new Execute(){
			public void execute() throws Exception
			{replaceKeyTabValue();}
		});
		add(vec,"Complete: key(tab)value",new Execute(){
			public void execute() throws Exception
			{completeKeyTabValue();}
		});
		add(vec,"Replace: key=value",new Execute(){
			public void execute() throws Exception
			{replaceKeyEqValue();}
		});
		add(vec,"Complete: key=value",new Execute(){
			public void execute() throws Exception
			{completeKeyEqValue();}
		});
		add(vec,"Complete: keys",new Execute(){
			public void execute() throws Exception
			{completeKeys();}
		});
		add(vec,"Export properties to Excel file",new Execute(){
			public void execute() throws Exception
			{exportToExcel();}
		});
		return vec;
    }
    
    
    
    
    
    
    
    
    
    
    
    private void add(Vector vec, String s, Execute e)
    {
    	vec.add(s);
    	actions.put(s,e);
    }
    
    
    private void println(String s)
    {textArea.append(s+"\n");}
    
    private void resetArea()
    {textArea.setText("");}
    
    
    
    
    
    
    /*
     * ACTIONS DISPLAY
     */
    
    
    private void displayKeys()
    {
    	resetArea();
    	Iterator it = map.keySet().iterator();
    	while(it.hasNext())
    	{
    		String key = (String) it.next();
    		println(key);
    	}
    }
    
    
    private void displayValues()
    {
    	resetArea();
    	Iterator it = map.values().iterator();
    	while(it.hasNext())
    	{
    		String value = (String) it.next();
    		println(value);
    	}
    }
    
    
    private void displayKeyEqValue()
    {
    	resetArea();
    	Iterator it = map.keySet().iterator();
    	while(it.hasNext())
    	{
    		String key = (String) it.next();
    		String value = (String) map.get(key);
    		println(key+" = "+value);
    	}
    }
    
    
    
    private void displayKeyTabValue()
    {
    	resetArea();
    	Iterator it = map.keySet().iterator();
    	while(it.hasNext())
    	{
    		String key = (String) it.next();
    		String value = (String) map.get(key);
    		println(key+"\t"+value);
    	}
    }
    
    
    
    
    
    

    
    
    /*
     * ACTIONS MODIFY
     */
    
    
    private void emptyValues()
    {
    	Iterator it = map.keySet().iterator();
    	while(it.hasNext())
    	{
    		String key = (String) it.next();
    		map.put(key,"");
    	}
    }
    
    
    
    private void clearProperties()
    {map.clear();}
    
    
    
    private void invertKeyValue()
    {
    	HashMap m = new HashMap();
    	Iterator it = map.keySet().iterator();
    	while(it.hasNext())
    	{
    		String key = (String) it.next();
    		String value = (String) map.get(key);
    		
    		if(m.containsKey(value))
    		{
    			int n = 1;
    			while(m.containsKey(value+"_"+n)) n++;
    			value = value+"_"+n;
    		}
    		m.put(value,key);
    	}
    	map.clear();
    	map.putAll(m);
    }
    
    
    
    
    private void encryptProp() throws Exception
    {
    	Map m = (Map) pbeEncrypt.transform(map);
    	map.clear();
    	map.putAll(m);
    }
    
    
    
    private void decryptProp() throws Exception
    {
    	Map m = (Map) pbeDecrypt.transform(map);
    	map.clear();
    	map.putAll(m);
    }
    
    
    
    
    
    
    /*
     * ACTIONS EXPORT IMPORT
     */
    
    
    private void exportToExcel()
    {
    	try
    	{
    		String[][] table = (String[][]) mapToTable.transform(map);
    		saveData.give(table);
    	}
    	catch(Exception e)
    	{Outside.err(this,"exportToExcel()",e);}
    }
    
    
    
    private void completeKeys()
    {
    	String text = textArea.getText();
    	String[] lines = text.split("[\n\r]+");
    	
    	for(int i=0;i<lines.length;i++)
    	{
    		String newKey = lines[i];
    		if(!newKey.equals("") && !map.containsKey(newKey))
    			map.put(newKey,"");
    	}
    }
    
    
    private void completeKeyTabValue()
    {
    	String text = textArea.getText();
    	String[] lines = text.split("[\n\r]+");
    	
    	for(int i=0;i<lines.length;i++)
    	{
    		String[] k = lines[i].split("\t",2);
    		String key = k[0];
    		String value = k[1];
    		
    		if(!key.equals("") && !map.containsKey(key))
    			map.put(key,value);
    	}
    }
    
    
    
    private void completeKeyEqValue()
    {
    	String text = textArea.getText();
    	String[] lines = text.split("[\n\r]+");
    	
    	for(int i=0;i<lines.length;i++)
    	{
    		String[] k = lines[i].split(" *= *",2);
    		String key = k[0];
    		String value = k[1];
    		
    		if(!key.equals("") && !map.containsKey(key))
    			map.put(key,value);
    	}
    }
    
    
    
    
    private void replaceKeyTabValue()
    {
    	String text = textArea.getText();
    	String[] lines = text.split("[\n\r]+");
    	
    	for(int i=0;i<lines.length;i++)
    	{
    		String[] k = lines[i].split("\t",2);
    		String key = k[0];
    		String value = k[1];
    		
    		if(!key.equals(""))
    		map.put(key,value);
    	}
    }
    
    
    
    private void replaceKeyEqValue()
    {
    	String text = textArea.getText();
    	String[] lines = text.split("[\n\r]+");
    	
    	for(int i=0;i<lines.length;i++)
    	{
    		String[] k = lines[i].split(" *= *",2);
    		String key = k[0];
    		String value = k[1];
    		
    		if(!key.equals(""))
    		map.put(key,value);
    	}
    }
    
    
}

