package gus05.entity.gus.env.config.filesverifier.installer;


import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellRenderer;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Give;
import gus05.framework.resources.IconProvider;



public class FilesVerifierInstaller implements Entity, Give, Runnable {


    public String getName()				{return "gus.env.config.filesverifier.installer";}
    public String getCreationDate()		{return "2009.02.26";}
    
    
    public static final String STATUS_WAITING = "WAITING";
    public static final String STATUS_PREPARING = "PREPARING";
    public static final String STATUS_PREPARED = "PREPARED";
    public static final String STATUS_FAILED = "FAILED";
    
    public static final Dimension DIM = new Dimension(500,250);
    public static final int BORDER_SIZE = 10;
    public static final int COLUMN_SIZE = 50;
    
    
    
    private Service prepareFile;
    private Service copyFiles;
    private Service tableTooltip;
    private Service isOnline;
    private Service progress;
    private Service popup;
    
    private IconProvider iconProvider;
    
    private Map map_file1_rule;
    private Map map_file1_file0;
    private ArrayList keys_files;
    
    private File storeDir;
    
    private int index;
    private boolean stop;
    private File file1;
    private File file0;
    private String rule;
    
    private JTable table;
    private TableModel0 model;
    private JPanel panel;
    
    private JButton button_ok;
    private JButton button_interrupt;
    private JLabel label_title;
    private JLabel label_info;
    
    
    
    private Icon icon(String key)
    {return iconProvider.getIcon(key);}
    
    
    

    public FilesVerifierInstaller() throws Exception
    {
    	prepareFile = Outside.service(this,"gus.env.config.filesverifier.preparefile");
    	copyFiles = Outside.service(this,"gus.file.treatment.filecopy2.multi");
    	tableTooltip = Outside.service(this,"gus.graphic.table.tooltip");
    	isOnline = Outside.service(this,"gus.watching.online");
    	progress = Outside.service(this,"gus.graphic.progressbar.progress1");
    	popup = Outside.service(this,"gus.graphic.dialog.popup0.blocked");
    	
    	storeDir = (File) Outside.resource(this,"defaultdir");
    	iconProvider = (IconProvider) Outside.resource(this,"app.gui.icons");
    	
    	map_file1_rule = new HashMap();
    	map_file1_file0 = new HashMap();
    	keys_files = new ArrayList();
    	
    	model = new TableModel0();
    	table = new JTable(model);
    	table.setDefaultRenderer(String.class,new TableCellRenderer0());
    	tableTooltip.give(table);
    	
    	column(1,COLUMN_SIZE);
    	column(2,COLUMN_SIZE);
    	
    	button_ok = new JButton("Copy files");
    	button_ok.setEnabled(false);
    	button_ok.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {ok();}
		});
    	button_interrupt = new JButton("Interrupt");
    	button_interrupt.setEnabled(false);
    	button_interrupt.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {stop();}
		});
    	
    	label_title = new JLabel("RESOURCES INSTALLER");
    	label_title.setHorizontalAlignment(JLabel.CENTER);
    	label_title.setOpaque(true);
    	label_title.setFont(label_title.getFont().deriveFont((float)15).deriveFont(Font.BOLD));
    	label_title.setForeground(Color.WHITE);
    	label_title.setBackground(Color.GRAY);
    	
    	label_info = new JLabel(" ");
    	label_info.setHorizontalAlignment(JLabel.CENTER);
    	label_info.setFont(label_info.getFont().deriveFont(Font.ITALIC));
    	
    	
    	JPanel panel_center = new JPanel(new BorderLayout());
    	panel_center.add(new JScrollPane(table),BorderLayout.CENTER);
    	panel_center.add(progress.gui(),BorderLayout.SOUTH);
    	panel_center.setBorder(BorderFactory.createEmptyBorder(BORDER_SIZE,0,0,0));
    	
    	JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
    	buttonPanel.add(label_info);
        buttonPanel.add(button_interrupt);
        buttonPanel.add(button_ok);
        
        
    	panel = new JPanel(new BorderLayout());
    	panel.add(label_title,BorderLayout.NORTH);
    	panel.add(panel_center,BorderLayout.CENTER);
    	panel.add(buttonPanel,BorderLayout.SOUTH);
    	panel.setBorder(BorderFactory.createEmptyBorder(BORDER_SIZE,BORDER_SIZE,BORDER_SIZE,BORDER_SIZE));
    	
    	
    	progress.register("color",Color.ORANGE.darker());
    	prepareFile.register("progress",progress);
    	
    	isOnline.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{table.repaint();}
		});
    }
    
    
    
    
    private void column(int index, int size)
    {
    	table.getColumnModel().getColumn(index).setMinWidth(size);
    	table.getColumnModel().getColumn(index).setMaxWidth(size);
    }

    
    
    
    

    public void give(Object obj) throws Exception
    {
    	map_file1_rule = (Map) obj;
    	keys_files = new ArrayList(map_file1_rule.keySet());
    	Collections.sort(keys_files);
    	
    	label_info.setText(" preparing files, please wait  ");
    	button_ok.setEnabled(false);
    	
    	index = 0;
    	stop = false;
    	model.fireTableDataChanged();
    	
    	popup.register("dimension",DIM);
    	new Thread(this).start();
    	popup.give(panel);
    	
    	Map map_file0_file1 = invMap(map_file1_file0);
    	copyFiles.give(map_file0_file1);
    }
    
    
    
    
    public void run()
    {
    	try
    	{
    		while(!popup.filter("isVisible"))
    		{Thread.yield();}
    		button_interrupt.setEnabled(true);
        	
    		for(int i=0;i<keys_files.size();i++)
        	{
    			index = i;
        		file1 = (File) keys_files.get(i);
        		handleFile();
        		if(stop) continue;
        	}
        	file0 = null;
        	file1 = null;
        	index = keys_files.size();
        	model.fireTableDataChanged();
        	
        	Set set_NotFoundFile0 = buildSet_NotFoundFile0();
        	if(!set_NotFoundFile0.isEmpty())
        	{
        		// preparation incomplete
        		model.fireTableDataChanged();
        	}
        	
        	button_interrupt.setEnabled(false);
        	button_ok.setEnabled(true);
        	label_info.setText(" file preparation complete !  ");
    	}
    	catch(Exception e)
    	{Outside.err(this,"run()",e);}
	}
    
    
    
    
    

    private void handleFile()
    {
    	try
    	{
    		if(stop) return;
    		rule = (String) map_file1_rule.get(file1);
    		file0 = new File(storeDir,file1.getName());

    		map_file1_file0.put(file1,file0);
    		model.fireTableDataChanged();
    		
    		if(!file0.exists())
    		prepareFile.register(rule,file0);
    	}
    	catch(Exception e)
    	{Outside.err(this,"handleFile()",e);}
    }

    
    
    
    
    
    
    
    
    private void ok()
    {
    	try
    	{
    		popup.give(null);
    	}
    	catch(Exception e)
    	{Outside.err(this,"ok()",e);}
    }
    
    
    
    private void stop()
    {
    	try
    	{
    		stop = true;
    		popup.give(null);
    	}
    	catch(Exception e)
    	{Outside.err(this,"stop()",e);}
    }
    
    
    
    
    
    
    private Set buildSet_NotFoundFile0()
    {
    	HashSet set = new HashSet();
    	Iterator it = map_file1_file0.keySet().iterator();
    	while(it.hasNext())
    	{
    		File f1 = (File) it.next();
        	File f0 = (File) map_file1_file0.get(f1);
        	if(!f0.exists()) set.add(f0);
    	}
    	return set;
    }
    
    
    
    
    private Map invMap(Map map)
    {
    	HashMap inv = new HashMap();
    	Iterator it = map.keySet().iterator();
    	while(it.hasNext())
    	{
    		Object key = it.next();
    		Object value = map.get(key);
    		inv.put(value,key);
    	}
    	return inv;
    }
    
    
    
   
    
    
    
    private String source(File f)
    {return (String) map_file1_rule.get(f);}
    
    
    
    private String status(File f)
    {
    	if(f==file1) return STATUS_PREPARING;
    	if(!map_file1_file0.containsKey(f)) return STATUS_WAITING;

    	File file0 = (File) map_file1_file0.get(f);
    	if(file0.exists()) return STATUS_PREPARED;
    	return STATUS_FAILED;
    }
    
    
    
    
    private boolean isOnline()
    {
    	try{return isOnline.filter("isOnline");}
    	catch(Exception e)
    	{Outside.err(this,"isOnline()",e);}
    	return false;
    }
    
    
    private Icon sourceIcon(String source)
    {
    	if(source.startsWith("url#"))return isOnline()?icon("WEB_online2"):icon("WEB_offline2");
    	if(source.startsWith("path#"))return icon("FILE_dir_");
    	if(source.startsWith("inside#"))return icon("FILE_jar");
    	return icon("");
    }
    
    
    
    private Icon statusIcon(String status)
    {
    	if(status.equals(STATUS_PREPARING)) return icon("UTIL_performing");
    	if(status.equals(STATUS_PREPARED)) return icon("UTIL_valid");
    	if(status.equals(STATUS_FAILED)) return icon("UTIL_invalid");
    	return null;
    }
    
    
    
    
    
    
    
    
    private class TableModel0 extends AbstractTableModel
    {
		public int getColumnCount() {return 3;}
		public int getRowCount() {return keys_files.size();}
		public Class getColumnClass(int y) {return String.class;}
		
		public String getColumnName(int y)
		{
			if(y==0) return "File";
			if(y==1) return "Source";
			return "Status";
		}

		public Object getValueAt(int x, int y)
		{
			File f = (File) keys_files.get(x);
			if(y==0) return f.getName();
			if(y==1) return source(f);
			return status(f);
		}	
    }
    
    
    
    
    
    
    
    
    
    private class TableCellRenderer0 extends JLabel implements TableCellRenderer
    {
    	private Font f_bold;
    	private Font f_plain;
    	
    	public TableCellRenderer0()
    	{
    		setOpaque(true);
    		setBackground(Color.WHITE);
    		
    		f_bold = getFont().deriveFont(Font.BOLD);
    		f_plain = getFont().deriveFont(Font.PLAIN);
    	}
    	
		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,int column)
		{
			String s = (String) value;
			
			if(row<index)
			{
				setForeground(Color.BLACK);
				setFont(f_plain);
			}
			else if(row==index)
			{
				setForeground(Color.BLACK);
				setFont(f_bold);
			}
			else if(row>index)
			{
				setForeground(Color.GRAY);
				setFont(f_plain);
			}
			
			if(column==0)
			{
				setHorizontalAlignment(JLabel.LEFT);
				setIcon(null);
				setText(s);
			}
			else if(column==1)
			{
				setHorizontalAlignment(JLabel.CENTER);
				setIcon(sourceIcon(s));
				setText("");
			}
			else if(column==2)
			{
				setHorizontalAlignment(JLabel.CENTER);
				setIcon(statusIcon(s));
				setText("");
			}
			return this;
		}
    }
    
}

