
package gus05.entity.gus.file.editing.editorholder2;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Hashtable;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.Scrollable;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Execute;
import gus05.framework.features.Give;
import gus05.framework.features.Graphic;
import gus05.framework.features.Register;
import gus05.framework.features.Retrieve;
import gus05.framework.tools.DefaultSupport;


public class EditorHolder2 extends DefaultSupport implements Entity, Graphic, Give, Register, Retrieve, Execute, ActionListener {


    public String getName()				{return "gus.file.editing.editorholder2";}
    public String getCreationDate()		{return "2008.06.06";}
    

    public static final String DEFAULTNAME = "gus.file.editor.fileinfo.editor1";
    
    
    private Service editor;
    private Service shift;
    
    private JScrollPane scroll;
    private JPanel panel;
    
    private String editorName;
    
    private Hashtable editorObjects;
    private Hashtable editorNames;
    
    
    

    public EditorHolder2() throws Exception
    {
        shift = Outside.service(this,"gus.graphic.panel.shiftpanel");
        
        editorObjects = new Hashtable();
        editorNames = new Hashtable();
        init();
        
        scroll = new JScrollPane();
        panel = (JPanel)shift.transform(new Graphic(){
            public JComponent gui() {return getEditorGui();}});
    }
    
    
    
    private void init()
    {
    	put("jpg","gus.graphic.panel.screen.image");
        put("gif","gus.file.editor.gif.editor1");
        put("bmp","gus.file.editor.bmp.editor1");
        put("htm","gus.file.editor.html.editor1");
        put("html","gus.file.editor.html.editor1");
        put("rtf","gus.file.editor.rtf.editor1");
        put("zip","gus.file.editor.zip.editor2");
        put("jar","gus.file.editor.jar.editor1");
        put("properties","gus.file.editor.prop.editor4");
        put("txt","gus.file.editor.string.editor1");
        put("java","gus.file.editor.string.editor1");
        put("log","gus.file.editor.string.editor1");
        put("csv","gus.file.editor.string.csv.editor1");
        put("lnk","gus.file.editor.string.lnk.editor1");
    }
    
    
    private void put(String key, String value)
    {editorNames.put(key,value);}
    
    
    
    public JComponent gui()
    {return panel;}
    
    
    public void register(String key, Object obj) throws Exception
    {if(editor!=null)editor.register(key,obj);}
    
    
    public void execute() throws Exception
    {if(editor!=null)editor.execute();}
    
    
    
    public Object retrieve(String key) throws Exception
    {
    	if(key.equals("editorName")) return editorName;
    	if(editor!=null)return editor.retrieve(key);
    	return null;
	}
    
    
    
    public void give(Object obj) throws Exception
    {
        File file = (File)obj;
        String editorName0 = findEditorName(file);
        if(editorName==null || !editorName.equals(editorName0))
            changeEditor(editorName0);
        
        editor.give(file);
    }
    
    
    
    private JComponent getEditorGui()
    {
        if(editor==null)return null;
        JComponent comp = editor.gui();
        if(comp==null)return null;
        
        if(comp instanceof Scrollable)
        {
            scroll.setViewportView(comp);
            return scroll;
        }
        return comp;
    }

    
    
    
    

    private void changeEditor(String newName) throws Exception
    {
        editorName = newName;
        if(editor!=null) editor.removeActionListener(this);
        
        if(editorObjects.containsKey(newName)) 
            editor = (Service) editorObjects.get(newName);
        else
        {
            editor = Outside.service(this,editorName);
            editorObjects.put(editorName,editor);
        } 
        editor.addActionListener(this);
        ((ActionListener)panel).actionPerformed(null);
        editorChanged();
    }
    
    
    
    
    private void editorChanged()
    {send(this,"editorChanged()");}
    
    public void actionPerformed(ActionEvent e)
    {send(this,e.getActionCommand());}
    
    
    
    
    
    private String findEditorName(File file)
    {
    	String ext = getFileExtension(file);
        
    	if(editorNames.containsKey(ext))
            return (String) editorNames.get(ext);
        return DEFAULTNAME;
    }
    
    
    
    private String getFileExtension(File file)
    {
        String[] n = file.getName().split("\\.");
        if(n.length>1) return n[n.length-1].toLowerCase();
        return "";
    }
}
