package gus05.entity.gus.file.zip.zipmaker1;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.features.Give;
import gus05.framework.features.Register;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;

import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;
import java.util.zip.ZipEntry;
import java.util.Enumeration;
import java.util.Vector;


public class ZipMaker1 implements Entity, Register, Give {
    
    
    public String getName()				{return "gus.file.zip.zipmaker1";}
    public String getCreationDate()		{return "2006.06.22";}
    
    
    public static int BUFFER = 2048;
    
    private PrintStream out;
    
    
    private File input;
    private File output;
    private int level = 7;
    
    
    
    public ZipMaker1() throws Exception
    {out = (PrintStream) Outside.resource(this,"out");}
    
    
    
    public void register(String key, Object obj) throws Exception
    {
        if(key.equals("input"))
        {registerInput((File)obj);return;}
        
        if(key.equals("output"))
        {registerOutput((File)obj);return;}
        
        if(key.equals("level"))
        {level = Integer.parseInt((String)obj);return;}
        
        if(key.equals("out"))
        {out = ((PrintStream)obj);return;}
        
        throw new Exception("Unknown key: "+key);
    }
    
    
    
    
    
    private void registerInput(File input)
    {
        if(!input.exists())
            out.println("Input does not exist: "+input);
        this.input = input;
    }
    
    
    private void registerOutput(File output)
    {
        this.output = output;
    }
    
    
    
    
    public void give(Object obj) throws Exception
    {
        if(input==null)		throw new Exception("Input is undefined");
        if(output==null)	throw new Exception("Output is undefined");
        
        String command = (String)obj;
        if(command.equals("zip")){zip();return;}
        if(command.equals("unzip")){unzip2();return;}
        
        throw new Exception("Unknown command: "+command);
    }
    
    
    
    
    
    
    
    
    private File[] directoryContent(File dir)
    {
        Vector vec = new Vector();
        File[] child = dir.listFiles();
        for(int i=0;i<child.length;i++)
        buildDirectoryContent(vec, child[i]);
        
        
        File[] content = new File[vec.size()];
        for(int i=0;i<vec.size();i++)
            content[i] = (File)vec.get(i);
        return content;
    }
    
    
    private void buildDirectoryContent(Vector vec, File file)
    {
        //if(file.isFile())
        vec.add(file);
        
        if(file.isDirectory())
        {
            File[] child = file.listFiles();
            for(int i=0;i<child.length;i++)
            buildDirectoryContent(vec, child[i]);
        }
    }
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    private void addEntry(ZipOutputStream zos, File inputFile, String inputRoot)
    {
        try
        {
            String entryName = inputFile.getAbsolutePath().substring(inputRoot.length());
            entryName = entryName.replace(File.separatorChar,'/');
            
            if(entryName.startsWith("/"))
                entryName = entryName.substring(1);
            
            if(inputFile.isDirectory())
            if(!entryName.endsWith("/"))
                entryName = entryName+"/";
            
            out.println("Adding");
            out.println(" file="+inputFile);
            out.println(" entry="+entryName);
            out.println();
            
            ZipEntry entry = new ZipEntry(entryName);
            zos.putNextEntry(entry);
            
            if(inputFile.isFile())
            {
                int b;
                byte data[] = new byte[BUFFER];
                FileInputStream fis = new FileInputStream(inputFile);
                while((b = fis.read(data, 0, BUFFER)) != -1)
                {zos.write(data, 0, b);}
            }
            
            zos.closeEntry();
        }
        catch(Exception e)
        {
            Outside.err(this,"addEntry(ZipOutputStream,File,String)",e);
            out.println("Error occurred on file "+inputFile.getAbsolutePath());
            out.println(e.toString());
        }
    }
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    private void zip() throws Exception
    {     
        File[] inputFiles = null;
        String inputRoot = null;
        
        if(input.isFile())
        {
            inputFiles = new File[]{input};
            inputRoot = input.getParent();
        }
        if(input.isDirectory())
        {
            inputFiles = directoryContent(input);
            inputRoot = input.getParent();
        }
        
        FileOutputStream fos = new FileOutputStream(output);
        BufferedOutputStream bos = new BufferedOutputStream(fos);
        ZipOutputStream zos = new ZipOutputStream(bos);
        
        zos.setMethod(ZipOutputStream.DEFLATED);
        zos.setLevel(level);
        
        out.println("Input root= "+inputRoot);
        
        for(int i=0; i<inputFiles.length; i++)
            addEntry(zos,inputFiles[i],inputRoot);
        
        zos.finish();
        zos.close();
    }
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    /*
     * CETTE METHODE EST CORROMPUE, POURQUOI ???
     * LA METHODE unzip2() FONCTIONNE
     */
    private void unzip() throws Exception
    {
        if(!input.isFile())
            throw new Exception("Invalid input: "+input);
        if(!output.isDirectory())
            throw new Exception("Invalid output: "+output);
        
        FileInputStream fis = new FileInputStream(input);
        BufferedInputStream bis = new BufferedInputStream(fis);
        ZipInputStream zis = new ZipInputStream(bis);
        
        ZipEntry entry;
        while((entry = zis.getNextEntry()) != null)
        { 
            File entryPath = new File(output,entry.getName());
            
            if(entry.isDirectory())
            {
                out.println("Extracting");
                out.println(" dir="+entryPath);
                out.println(" entry="+entry);
                out.println();
                
                entryPath.mkdirs();
            }
            else
            {
                out.println("Extracting");
                out.println(" file="+entryPath);
                out.println(" entry="+entry);
                out.println();
                
                if(!entryPath.getParentFile().exists())
                    entryPath.getParentFile().mkdirs();

                FileOutputStream fos = new FileOutputStream(entryPath);
                BufferedOutputStream bos = new BufferedOutputStream(fos,BUFFER);

                int b;
                byte data[] = new byte[BUFFER];
                while ((b = zis.read(data, 0, BUFFER)) != -1) 
                {bos.write(data, 0, BUFFER);}
                
                bos.flush();
                bos.close();
            }    
        }
        zis.close();
    }
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    private void unzip2() throws Exception
    {
        if(!input.isFile())
            throw new Exception("Invalid input: "+input);
        if(!output.isDirectory())
            throw new Exception("Invalid output: "+output);
        
        ZipFile zipFile = new ZipFile(input);
        Enumeration entries = zipFile.entries();
        
        while(entries.hasMoreElements())
        {
            ZipEntry entry = (ZipEntry)entries.nextElement();
            File entryPath = new File(output,entry.getName());
            extractEntry(zipFile,entry,entryPath);
        }
        zipFile.close();
    }
    
    
    
    
    
    
    

    private void extractEntry(ZipFile zipFile, ZipEntry entry, File entryPath)
    {
    	try
    	{
    		if(entry.getSize()<0)
    			throw new Exception("invalid entry detected: "+entry.getName()+" (size="+entry.getSize()+")");
    		
    		entryPath.getParentFile().mkdirs();

    		out.println("Extracting");
    		out.println(" file="+entryPath);
    		out.println(" entry="+entry);
    		out.println();

    		if(entry.isDirectory())
    		{
    			entryPath.mkdir();
    			return;
    		}
    		FileOutputStream fos = new FileOutputStream(entryPath);
    		BufferedOutputStream bos = new BufferedOutputStream(fos);
    		InputStream is = zipFile.getInputStream(entry);
    		
    		copyInputStream(is,bos);
    	}
    	catch(Exception e)
    	{
    		Outside.err(this,"extractEntry(ZipFile,ZipEntry,File)",e);
    		out.println("Error occurred on entry "+entry.getName());
    		out.println(e.toString());
    	}
    }



    
    
    public static final void copyInputStream(InputStream in, OutputStream out) throws IOException
    {
        byte[] buffer = new byte[1024];
        int len;

        while((len = in.read(buffer)) >= 0)
            out.write(buffer, 0, len);

        in.close();
        out.close();
    }
    
    
    
    
}
