package gus05.entity.gus.graphic.textcomp.action.q.selectionfocus;


import java.awt.Event;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.Stack;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.KeyStroke;
import javax.swing.text.JTextComponent;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.features.Execute;
import gus05.framework.features.Give;
import gus05.framework.features.Transform;
import gus05.framework.resources.ActionBuilder;



public class SelectionFocus implements Entity, Give, Transform {


    public String getName()				{return "gus.graphic.textcomp.action.q.selectionfocus";}
    public String getCreationDate()		{return "2006.09.25";}



    private ActionBuilder actionBuilder;
    private KeyStroke focusKey;
    
    public SelectionFocus() throws Exception
    {
        actionBuilder = (ActionBuilder) Outside.resource(this,"app.actionbuilder");
        focusKey = KeyStroke.getKeyStroke(KeyEvent.VK_Q,Event.CTRL_MASK);
    }


    
    public Object transform(Object obj) throws Exception
	{
		JTextComponent comp = (JTextComponent)obj;
		FocusHandler handler = new FocusHandler(comp);
		
		Vector vec = new Vector();
		vec.add(handler.getFocusIn());
		vec.add(handler.getFocusOut());
		
		return vec;
	}



    public void give(Object obj) throws Exception
    {
        JTextComponent comp = (JTextComponent)obj;
        FocusHandler handler = new FocusHandler(comp);
    }

    
    
    
    
    
    
    private class FocusHandler
    {
        private JTextComponent comp;
        
        private Action actionFocusIn;
        private Action actionFocusOut;
        
        public Action getFocusIn()
        {return actionFocusIn;}
        
        public Action getFocusOut()
        {return actionFocusOut;}
        
        private Stack rightPart;
        private Stack leftPart;
        
        
        public FocusHandler(JTextComponent comp)
        {
            this.comp = comp;
            rightPart = new Stack();
            leftPart = new Stack();
            
            initActions();
            update();
        }
        
        
        
        
        private void initActions()
        {
            actionFocusIn = actionBuilder.build("TEXT_focusIn",new Execute(){
                public void execute() throws Exception {inside();}});
            
            actionFocusOut = actionBuilder.build("TEXT_focusOut",new Execute(){
                public void execute() throws Exception {inside();}});
            
            comp.getInputMap().put(focusKey, new AbstractAction(){
                public void actionPerformed(ActionEvent e){invert();} 
            });
        }
        
        
        
        
        private void inside()
        {
            String selected = comp.getSelectedText();
            if(selected==null) return;
            
            int p = comp.getCaretPosition();
            int s1 = comp.getSelectionStart();
            int s2 = comp.getSelectionEnd();
            
            String text = comp.getText();
            String left = text.substring(0,s1);
            String right = text.substring(s2,text.length());
            
            rightPart.push(right);
            leftPart.push(left);
            
            comp.setText(selected);
            comp.requestFocus();
            
            if(p<s1) comp.setCaretPosition(0);
            else if(p<s2) comp.setCaretPosition(p-s1);
            else comp.setCaretPosition(s2-s1);
            update();
        }
        
        
        
        private void outside()
        {
            if(rightPart.isEmpty()) return;
            int p = comp.getCaretPosition();
            
            String left = (String)leftPart.pop();
            String text = comp.getText();
            String right = (String)rightPart.pop();
            
            int s1 = left.length();
            int s2 = s1+text.length();
            
            comp.setText(left+text+right);
            update();
            
            comp.requestFocus();
            comp.select(s1,s2);
        }
        
        
        
        private void invert()
        {
            if(comp.getSelectedText()==null)outside();
            else inside();
        }
        
        
        
        private void update()
        {actionFocusOut.setEnabled(!rightPart.isEmpty());}        
    }

}

