package gus05.entity.gus.mail.account.mainaccount2;


import java.util.Date;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import javax.mail.Session;
import javax.mail.Store;
import javax.mail.Transport;
import javax.swing.JOptionPane;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Filter;
import gus05.framework.features.Retrieve;
import gus05.framework.tools.DefaultSupport;



public class MainAccount2 extends DefaultSupport implements Entity, Retrieve, Filter {


    public String getName()				{return "gus.mail.account.mainaccount2";}
    public String getCreationDate()		{return "2009.10.07";}

    
    private static final String KEY_LOGIN = "login";
    private static final String KEY_PASSWORD = "password";
    
    public static final int LAPSE = 5000;
    
    
    
    private Service watchingOnline;
    private Service askLoginPassword;
    private Service account_gmail;
    private Service account_cegetel;
    private Service account_hotmail;
    
    private Service currentAccount;
    
    private Transport transport;
    private Session transportSession;
    
    private Store store;
    private Session storeSession;
    
    private Map infoMap;
    private String login;
    private String password;

    
    private Timer timer;
    private TimerTask task;
    
    private boolean transportFailed = false;
    private boolean storeFailed = false;
    
    
    
    

    public MainAccount2() throws Exception
    {
    	watchingOnline = Outside.service(this,"gus.watching.online");
    	askLoginPassword = Outside.service(this,"gus.security.askinfo.loginpasswordinit1");
    	account_gmail = Outside.service(this,"gus.mail.account.mainaccount2.gmail");
    	account_cegetel = Outside.service(this,"gus.mail.account.mainaccount2.cegetel");
    	account_hotmail = Outside.service(this,"gus.mail.account.mainaccount2.hotmail");
    	
        infoMap = (Map) Outside.resource(this,"mailaccount.info");
        
        initLoginAndPassword();
        
        currentAccount = findCurrentAccount();
        currentAccount.register("login",login);
        currentAccount.register("password",password);
    }
    
    
    
    
    private void initLoginAndPassword() throws Exception
    {
    	if(!infoMap.containsKey(KEY_LOGIN) || !infoMap.containsKey(KEY_PASSWORD))
            askForConnectionInformation();
        
        if(!infoMap.containsKey(KEY_LOGIN)) throw new Exception("login information missing for connection");
        if(!infoMap.containsKey(KEY_PASSWORD)) throw new Exception("password information missing for connection");
        
        login = (String) infoMap.get(KEY_LOGIN);
        password = (String) infoMap.get(KEY_PASSWORD);
    }
    
    
    private void askForConnectionInformation() throws Exception
    {
    	String[] infos = (String[]) askLoginPassword.provide();
    	if(infos==null) return;
    	
    	String newLogin = infos[0];
    	String newPassword = infos[1];
    	
    	if(newLogin==null) return;
    	if(newPassword==null) return;
    	
    	infoMap.put(KEY_LOGIN,newLogin);
        infoMap.put(KEY_PASSWORD,newPassword);
    }
    
    
    
    
    private Service findCurrentAccount()throws Exception
    {
    	if(login.endsWith("@gmail.com")) return account_gmail;
    	if(login.endsWith("@cegetel.net")) return account_cegetel;
    	if(login.endsWith("@hotmail.com")) return account_hotmail;
    	
    	throw new Exception("Account not found for login: "+login);
    }
    
    
    
    private void initTimer()
    {
    	if(timer!=null) return;
    	timer = new Timer();
        task = new TimerTask(){public void run(){checkConnection();}};
        timer.schedule(task,new Date(),LAPSE);
    }
    
    
    
    
    
    private boolean online() throws Exception
    {return watchingOnline.filter("online");}
    
    

    
    
    
    
    
    
    public boolean filter(Object obj) throws Exception
	{
    	if(transportFailed) return false;
    	if(storeFailed) return false;
    	if(!online()) return false;
    	return true;
	}
    
    


    public Object retrieve(String key) throws Exception
    {
        if(key.equals("login")) return login;
        if(key.equals("transport")) return getTransport();
        if(key.equals("store")) return getStore();
        
        if(key.equals("session"))
        {getTransport(); return transportSession;}
        
        if(key.equals("transportSession"))
        {getTransport();return transportSession;}
        
        if(key.equals("storeSession"))
        {getStore();return storeSession;}
        
        if(key.equals("keys")) return new String[]{"login","transport","store","session","transportSession","storeSession"};
        throw new Exception("Unknown key: "+key);
    }
    
    
    
    
    
    
    

    private Transport getTransport() throws Exception
    {
    	if(transport == null) buildTransport();
    	return transport;
    }
    
    
    private void buildTransport() throws Exception
    {
    	if(!online() || transportFailed) return;
    	
    	try
    	{
    		transport = (Transport) currentAccount.retrieve("transport");
    		transportSession = (Session) currentAccount.retrieve("transportSession");
        	initTimer();
        	connected();
    	}
    	catch(Exception e)
    	{
    		Outside.err(this,"buildTransport()",e);
    		transportFailed = true;
    		aborted();
    		throw new Exception("transport connection failed for mailbox: "+login+"\n"+e.toString());
    	}
    }

    

    
    
    
    
    
    private Store getStore() throws Exception
    {
    	if(store == null) buildStore();
    	return store;
    }
    
    
    private void buildStore() throws Exception
    {
    	try
    	{
    		if(storeFailed || !online()) return;
        	store = (Store) currentAccount.retrieve("store");
        	storeSession = (Session) currentAccount.retrieve("storeSession");
        	initTimer();
        	connected();
    	}
    	catch(Exception e)
    	{
    		Outside.err(this,"buildStore()",e);
    		storeFailed = true;
    		aborted();
    		throw new Exception("store connection failed for mailbox: "+login+"\n"+e.toString());
    	}
    }
    
    
    
    private void checkConnection()
    {
    	try
    	{
    		if(storeFailed || transportFailed || !online()) return;
    		currentAccount.execute();
    		connected();
    	}
    	catch(Exception e)
    	{
    		Outside.err(this,"checkConnection()",e);
    		transportFailed = true;
    		storeFailed = true;
    		aborted();
    	}
    }
    
    
	
    
    
    
    private void connected()
    {send(this,"connected()");}
    
    private void aborted()
    {send(this,"aborted()");}

}

