package gus05.entity.gus.mail.send.autosender;


import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import java.net.URL;
import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Execute;
import gus05.framework.features.Filter;
import gus05.framework.features.Register;
import gus05.framework.tools.DefaultSupport;



public class MailAutoSender extends DefaultSupport implements Entity, Execute, Register, Filter {


    public String getName()				{return "gus.mail.send.autosender";}
    public String getCreationDate()		{return "2007.12.18";}

    
    public static final long DELAY = 10000;
    public static final String URLTEST = "http://www.google.com";
    

    private Service sendMail;
    
    private File storeDir;
    private File failedDir;
    private File sentDir;
    
    private Timer timer;
    private TimerTask task;
    private URL urlTest;
    private PrintStream out;
    
    private boolean activated = true;
    private boolean checking = false;
    
    

    
    public MailAutoSender() throws Exception
    {
        sendMail = Outside.service(this,"gus.mail.send.mail1");
        
        storeDir = (File) Outside.resource(this,"storeDir");
        failedDir = (File) Outside.resource(this,"failedDir");
        sentDir = (File) Outside.resource(this,"sentDir");
        out = (PrintStream) Outside.resource(this,"out");
        
        urlTest = new URL(URLTEST);
        
        
        task = new TimerTask(){
            public void run() {check();}
        };
        
         Outside.service(this,"app.supportstart").addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {initTimer();}});
    }
    
    
    
    
    private void initTimer()
    {
    	timer = new Timer();
    	timer.schedule(task,new Date(),DELAY);
    }
    
    
    
    public void execute() throws Exception
    {check();}
    
    
    
    public boolean filter(Object obj) throws Exception
    {return activated;}
	
    
    
    
    public void register(String key, Object obj) throws Exception
    {
		if(key.equals("storeDir")) {storeDir = (File) obj;return;}
		if(key.equals("failedDir")) {failedDir = (File) obj;return;}
		if(key.equals("sentDir")) {sentDir = (File) obj;return;}
		if(key.equals("activate")) {activate();return;}
		if(key.equals("disactivate")) {disactivate();return;}
		
		throw new Exception("Unknown key: "+key);
	}
    
    
    private void activate()
    {
    	initTimer();
    	activated = true;
    	send(this,"activate()");
    }
    
    private void disactivate()
    {
    	activated = false;
    	send(this,"disactivate()");
    }

    
    
    
    
    private boolean mailIsReady()
    {return storeDir.list().length>0;}
    
    
    
    private boolean isOnline()
    {
        try
        {
            urlTest.openConnection().connect();
            return true;
        }
        catch(IOException e)
        {return false;}
    }
    
    
    


    private void check()
    {
    	if(!activated) return;
    	if(checking) return;
    	
    	checking = true;
        if(mailIsReady() && isOnline())
        {
        	out.println("checking for mails to be sent");
            File[] f = storeDir.listFiles();
            if(f.length>0) out.println(f.length+" mails have been found");
            for(int i=0;i<f.length;i++) handleFile(f[i]);
        }
        checking = false;
    }
    
    
    
    
    private void handleFile(File file)
    {
    	boolean sent = sendMail(file);
    	if(sent)
    	{
    		moveToDir(sentDir,file);
    		mailSent();
    	}
    	else
    	{
    		moveToDir(failedDir,file);
    		mailFailed();
    	}
    }


    
    
    
    private void mailSent()
    {send(this,"mailSent()");}
    
    private void mailFailed()
    {send(this,"mailFailed()");}
    
    
    
    
    
    private boolean sendMail(File file)
    {
        try
        {
            sendMail.give(file);
            return true;
        }
        catch(Exception e)
        {
        	Outside.err(this,"sendMail(File)",e);
        	return false;
        }
    }
    
    
    
    
    
    
//    private void deleteFile(File file)
//    {
//    	try
//        {
//            boolean r = file.delete();
//            if(!r) throw new Exception("File "+file+" could not be deleted");
//        }
//        catch(Exception e)
//        {Outside.err(this,"deleteFile(File)",e);}
//    }
    
    
    
    
    
    private void moveToDir(File dir, File file)
    {
    	try
        {
    		File newFile = new File(dir,file.getName());
        	boolean r = file.renameTo(newFile);
        	if(!r) throw new Exception("File "+file+" could not be moved to "+dir);
        }
        catch(Exception e)
        {Outside.err(this,"moveToDir(File,File)",e);}
    }
    
    
    
    
}

