package gus05.entity.gus.mail.send.mailingsender.gui;


import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Graphic;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import java.util.Map;



public class MailingSenderGui implements Entity, Graphic, Runnable, ActionListener {


    public String getName()				{return "gus.mail.send.mailingsender.gui";}
    public String getCreationDate()		{return "2009.05.02";}


    private Service mailingSender;
    private Service buildMapGui;
    private Service progress;
    
    private JPanel panel;
    private JButton button;
    
    private boolean running = false;




    public MailingSenderGui() throws Exception
    {
    	mailingSender = Outside.service(this,"gus.mail.send.mailingsender");
    	buildMapGui = Outside.service(this,"gus.mail.send.mailingsender.gui.buildmap");
    	progress = Outside.service(this,"gus.graphic.progressbar.testprogress1");
    	
    	mailingSender.register("progress",progress);
    	
    	button = new JButton("Start mailing");
    	button.setForeground(Color.BLUE);
    	button.addActionListener(this);
    	
    	JPanel p_bottom = new JPanel(new GridLayout(2,1));
    	p_bottom.add(progress.gui());
    	p_bottom.add(button);
    	
        panel = new JPanel(new BorderLayout());
        panel.add(buildMapGui.gui(),BorderLayout.CENTER);
        panel.add(p_bottom,BorderLayout.SOUTH);
    }



    public JComponent gui()
    {return panel;}
    
    
    
    public void actionPerformed(ActionEvent e)
    {perform();}

    
    
    private void perform()
    {
    	if(running) interruptMailing();
    	else startMailing();
    }
    
    
    
    private void interruptMailing()
    {
    	try{mailingSender.register("stop",null);}
    	catch(Exception e)
    	{Outside.err(this,"interruptMailing()",e);}
    }
    
    
    
    private void startMailing()
    {
    	running = true;
    	button.setText("Interrupt mailing");
    	button.setForeground(Color.ORANGE);
    	new Thread(this).start();
    }
    
    
    
	public void run()
	{
		try
		{
			Map m = (Map) buildMapGui.provide();
			performMailing(m);
		}
		catch(Exception e)
		{Outside.err(this,"run()",e);}
		
		button.setText("Start mailing");
    	button.setForeground(Color.BLUE);
    	running = false;
	}
	
	
	
	private  void performMailing(Map m) throws Exception
	{
		if(m==null)	return;
		boolean r = confirmMailing(m);
    	if(!r) return;
    	
		mailingSender.give(m);
		messageEnd();
	}
	
	
	
	
	
	
	
	
	private boolean confirmMailing(Map m)
    {
		Object obj_to = m.get(MAIL.KEY_TO);
    	String q = description(obj_to);
    	int r = JOptionPane.showConfirmDialog(null,"You are about to send email(s) to "+q+"\nContinue ?");
    	return r==JOptionPane.YES_OPTION;
    }
	
	
	private void messageEnd()
	{
		JOptionPane.showMessageDialog(null,"The mailing is complete");
	}
    
    
    
    private String description(Object obj_to)
    {
    	if(obj_to instanceof String)
    		return (String) obj_to;
    	List list = (List) obj_to;
    	return list.size()+" persons";
    }
	

}

