package gus05.entity.gus.mail.send.mailsender.historic;


import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Graphic;
import gus05.framework.features.Support;
import gus05.framework.resources.IconProvider;

import javax.swing.ButtonGroup;
import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellRenderer;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.Map;



public class MailSenderHistoric implements Entity, Graphic, ActionListener, ListSelectionListener {


    public String getName()				{return "gus.mail.send.mailsender.historic";}
    public String getCreationDate()		{return "2009.05.02";}

    
    public static final String STATUS_MAYBE = "MAYBE";
    public static final String STATUS_ABSENCE = "ABSENCE";
    public static final String STATUS_READ = "READ";
    public static final String STATUS_FAILED = "FAILED";

    
    private Service mailSender;
    private Service tableTooltip;
    private IconProvider iconProvider;
    
    private Map storeMap;
    private Map statusMap;
    private ArrayList keys;
    
    private JPanel panel;
    private TableModel0 model;
    private JTable table;
    
    private JRadioButton radio_maybe;
    private JRadioButton radio_absence;
    private JRadioButton radio_read;
    private JRadioButton radio_failed;
    

    
    private Icon icon(String key)
    {return iconProvider.getIcon(key);}



    public MailSenderHistoric() throws Exception
    {
    	mailSender = Outside.service(this,"gus.mail.send.mailsender");
    	tableTooltip = Outside.service(this,"gus.graphic.table.tooltip");
    	iconProvider = (IconProvider) Outside.resource(this,"app.gui.icons");
    	
    	storeMap = (Map) mailSender.retrieve("storeMap");
    	statusMap = (Map) Outside.resource(this,"defaultdatamap");
    	keys = new ArrayList();
    	
    	model = new TableModel0();
    	table = new JTable(model);
    	table.setDefaultRenderer(String.class, new TableCellRenderer0());
    	table.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
    	
    	table.getSelectionModel().addListSelectionListener(this);
    	tableTooltip.give(table);
    	
    	columnLength(0,150);
    	columnLength(1,50);
    	
    	
    	radio_maybe = new JRadioButtonBold("Received ?",false);
    	radio_absence = new JRadioButtonBold("Absence",false);
    	radio_read = new JRadioButtonBold("Read",false);
    	radio_failed = new JRadioButtonBold("Failed",false);
    	
    	radio_maybe.setIcon(statusIcon(STATUS_MAYBE));
    	radio_absence.setIcon(statusIcon(STATUS_ABSENCE));
    	radio_read.setIcon(statusIcon(STATUS_READ));
    	radio_failed.setIcon(statusIcon(STATUS_FAILED));
    	
    	ButtonGroup group = new ButtonGroup();
    	group.add(radio_maybe);
    	group.add(radio_absence);
    	group.add(radio_read);
    	group.add(radio_failed);
    	
    	JPanel pp_bottom = new JPanel(new GridLayout(1,4));
    	pp_bottom.add(radio_maybe);
    	pp_bottom.add(radio_absence);
    	pp_bottom.add(radio_read);
    	pp_bottom.add(radio_failed);
    	
    	radio_maybe.addActionListener(this);
    	radio_absence.addActionListener(this);
    	radio_read.addActionListener(this);
    	radio_failed.addActionListener(this);
    	
    	JPanel p_bottom = new JPanel(new BorderLayout());
    	p_bottom.add(pp_bottom,BorderLayout.WEST);
    	
    	panel = new JPanel(new BorderLayout());
    	panel.add(new JScrollPane(table),BorderLayout.CENTER);
    	panel.add(p_bottom,BorderLayout.SOUTH);
    	
    	mailSender.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {updateGui();}
		});
    	updateGui();
    }



    public JComponent gui()
    {return panel;}


    
    public void actionPerformed(ActionEvent e)
    {statusEdited();}
    
    
    public void valueChanged(ListSelectionEvent e)
	{selectionChanged();}
    
    
    
    private void updateGui()
    {
    	keys = new ArrayList(storeMap.keySet());
    	Collections.sort(keys);
    	model.fireTableDataChanged();
    	setRadioEnabled(false);	
    }
    
    
    
    private void statusEdited()
    {
    	String newStatus = status();
    	int[] index = table.getSelectedRows();
    	
    	for(int i=0;i<index.length;i++)
    		setStatus(index[i],newStatus);
    	
    	model.fireTableRowsUpdated(index[0],index[index.length-1]);
    }
    
    
    
    private void setStatus(int index, String value)
    {
    	String key = (String) keys.get(index);
    	statusMap.put(key,value);
    }
    
    
    
    private void setRadioEnabled(boolean enabled)
    {
    	radio_maybe.setEnabled(enabled);
    	radio_absence.setEnabled(enabled);
    	radio_read.setEnabled(enabled);
    	radio_failed.setEnabled(enabled);
    }
    
    
    
    
    
    private void selectionChanged()
    {
    	
    	int index = table.getSelectedRow();
    	if(index<0)
    	{
    		setRadioEnabled(false);
    		return;
    	}

    	setRadioEnabled(true);
    	String key = (String) keys.get(index);
    	String status = getStatus(key);

    	if(status.equals(STATUS_MAYBE)) radio_maybe.setSelected(true);
    	if(status.equals(STATUS_ABSENCE)) radio_absence.setSelected(true);
    	if(status.equals(STATUS_READ)) radio_read.setSelected(true);
    	if(status.equals(STATUS_FAILED)) radio_failed.setSelected(true);
    }
    
    
    
    private void columnLength(int index, int length)
    {
    	table.getColumnModel().getColumn(index).setMinWidth(length);
    	table.getColumnModel().getColumn(index).setMaxWidth(length);
    }
    
    
    
    private String getStatus(String key)
    {
    	if(statusMap.containsKey(key))
			return (String) statusMap.get(key);
		return STATUS_MAYBE;
    }
    
    private String status()
    {
    	if(radio_maybe.isSelected()) return STATUS_MAYBE;
    	if(radio_absence.isSelected()) return STATUS_ABSENCE;
    	if(radio_read.isSelected()) return STATUS_READ;
    	if(radio_failed.isSelected()) return STATUS_FAILED;
    	return STATUS_MAYBE;
    }
    
    
    
    private Icon statusIcon(String status)
    {
    	if(status.equals(STATUS_ABSENCE)) return icon("MAIL_mailSent_Delivered_Absence");
    	if(status.equals(STATUS_MAYBE)) return icon("MAIL_mailSent_Delivered_Maybe");
    	if(status.equals(STATUS_READ)) return icon("MAIL_mailSent_Delivered_Read");
    	if(status.equals(STATUS_FAILED)) return icon("MAIL_mailSent_DeliveryFailed");
    	return icon("MAIL_mailSent_Delivered_Maybe");
    }
    
    
    private SimpleDateFormat sdf1 = new SimpleDateFormat("yyyyMMdd_HHmmss");
    private SimpleDateFormat sdf2 = new SimpleDateFormat("yyyy-MM-dd  HH:mm:ss");
    
    private String formatTime(String timeStamp)
    {
    	try
    	{
    		Date d = sdf1.parse(timeStamp);
    		return sdf2.format(d);
		}
    	catch(ParseException e)
		{
			Outside.err(this,"formatTime(String)",e);
			return timeStamp;
		}
    }
    
    
    
    
    
    
    
    private class TableModel0 extends AbstractTableModel
    {
		public int getColumnCount() {return 3;}
		public int getRowCount() {return keys.size();}
		public Class getColumnClass(int y) {return String.class;}
		
		public String getColumnName(int y)
		{
			if(y==0) return "Sending time";
			if(y==1) return "Status";
			return "Mail address";
		}
		
		public Object getValueAt(int x, int y)
		{
			String key = (String) keys.get(x);
			if(y==0) return key;
			if(y==1) return getStatus(key);
			return storeMap.get(key);
		}
    }
    
    
    
    
    
    
    private class TableCellRenderer0 extends JLabel implements TableCellRenderer
    {
    	private Font font_b;
    	private Font font_p;
    	
    	public TableCellRenderer0()
    	{
    		super();
    		setOpaque(true);
    		setBackground(Color.WHITE);
    		font_b = getFont().deriveFont(Font.BOLD);
    		font_p = getFont().deriveFont(Font.PLAIN);
    	}
    	
		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row, int column)
		{
			String s = (String) value;
			
			if(column==0)
			{
				setText(" "+formatTime(s));
				setIcon(null);
				setHorizontalAlignment(JLabel.LEFT);
			}
			else if(column==1) 
			{
				setText("");
				setIcon(statusIcon(s));
				setHorizontalAlignment(JLabel.CENTER);
			}
			else 
			{
				setText(" "+s);
				setIcon(icon("MAIL_mailSent"));
				setHorizontalAlignment(JLabel.LEFT);
			}

			if(isSelected)
			{
				setForeground(Color.GREEN);
				setFont(font_b);
			}
			else
			{
				setForeground(Color.BLACK);
				setFont(font_p);
			}
			return this;
		}
    }

}

