package gus05.entity.gus.management.gui.systemtray.init;


import java.awt.SystemTray;
import java.awt.TrayIcon;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowStateListener;
import java.io.PrintStream;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JPopupMenu;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Register;
import gus05.framework.features.Retrieve;



public class SystemTrayInit implements Entity, Register, Retrieve, MouseListener, WindowStateListener {


    public String getName()				{return "gus.management.gui.systemtray.init";}
    public String getCreationDate()		{return "2007.03.21";}


    
    private Service provideActionExit;
    
    private TrayIcon tray;
    private JFrame mainFrame;
    private String appTitle;
    private ImageIcon appIcon;
    private PrintStream out;
    private int activatedState;
    
    private Timer timer;
    private JPopupMenu popup;
    private Action actionExit;
    

    public SystemTrayInit() throws Exception
    {
        provideActionExit = Outside.service(this,"gus.app.action.exit");
        
        out = (PrintStream) Outside.resource(this,"out");
        mainFrame = (JFrame) Outside.resource(this,"app.gui.frame");
        appTitle = (String) Outside.resource(this,"app.title");
        appIcon = (ImageIcon) Outside.resource(this,"app.icon");
        
        if(!SystemTray.isSupported())
        {
            out.println("SystemTray is not available: entity disabled");
            return;
        }
        
        timer = new Timer();
        popup = new JPopupMenu();
        
        actionExit = (Action) provideActionExit.provide();
        popup.add(actionExit);
        
        tray = new TrayIcon(appIcon.getImage(),appTitle);
        tray.addMouseListener(this);
        
        if(mainFrame!=null)
        mainFrame.addWindowStateListener(this);
        
        Outside.service(this,"app.supportstart").addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){init();}
        });
    }
    
    
    private void init()
    {
        try{SystemTray.getSystemTray().add(tray);}
        catch(Exception e)
        {Outside.err(this,"init()",e);}
    }
    
    
    
    
    public void windowStateChanged(WindowEvent e)
    {
        if(e.getNewState()==1 || e.getNewState()==7)
        {
            if(mainFrame!=null) mainFrame.setVisible(false);
            activatedState = e.getOldState();
        }
    }
    
    
    
    
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    public void mousePressed(MouseEvent e) {maybeShowPopup(e);}
    public void mouseReleased(MouseEvent e) {maybeShowPopup(e);}
    public void mouseClicked(MouseEvent e)
    {
        if(e.getButton()==MouseEvent.BUTTON1)
            changeActivation();
    }

    
    
    
    
    private void changeActivation()
    {
        if(mainFrame==null) return;
        if(!mainFrame.isVisible()) activate();
        else disactivate();
    }
    
    private void activate()
    {
        mainFrame.setVisible(true);
        mainFrame.setExtendedState(activatedState);
        mainFrame.toFront();
    }
    
    private void disactivate()
    {
        activatedState = mainFrame.getExtendedState();
        mainFrame.setVisible(false);
    }
    
    
    
    
    
    
    private void maybeShowPopup(MouseEvent e)
    {
        if(!e.isPopupTrigger()) return;
        
        popup.show(e.getComponent(),e.getX(),e.getY());
        
        TimerTask t = new TimerTask(){public void run() {popup.setVisible(false);}};
        timer.schedule(t,2000);
    }
    
    
    
    public void register(String key, Object obj) throws Exception 
    {
        String[] n = (String[])obj;
        if(key.equals("E")) {displayErrorMessage(n[0],n[1]);return;}
        if(key.equals("W")) {displayWarningMessage(n[0],n[1]);return;}
        if(key.equals("M")) {displayInfoMessage(n[0],n[1]);return;}
        
        throw new Exception("Unknown key: "+key);
    }
    
    
    
    private void displayErrorMessage(String title, String message)
    {tray.displayMessage(title,message,TrayIcon.MessageType.ERROR);}
    
    private void displayWarningMessage(String title, String message)
    {tray.displayMessage(title,message,TrayIcon.MessageType.WARNING);}
    
    private void displayInfoMessage(String title, String message)
    {tray.displayMessage(title,message,TrayIcon.MessageType.INFO);}
    
    
    
    
    
    public Object retrieve(String key) throws Exception
    {
        if(key.equals("tray")) return tray;
        if(key.equals("popup")) return popup;
        if(key.equals("keys")) return new String[]{"tray","popup"};
        
        throw new Exception("Unknown key: "+key);
    }
    
    
}

