package gus05.entity.gus.web.download.filedownloader.auth1;


import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Execute;
import gus05.framework.features.Register;
import gus05.framework.tools.DefaultSupport;



public class FileDownloaderAuth1 extends DefaultSupport implements Entity, Register, Runnable {


    public String getName()				{return "gus.web.download.filedownloader.auth1";}
    public String getCreationDate()		{return "2009.01.21";}


    
    private Service setAuth1;
    private File file;
    private URL url;
    private String login;
    private String password;
    
    private boolean stop;
    
    private Register progress_reg;
    private Execute progress_e;

    
    
    public FileDownloaderAuth1() throws Exception
    {
    	setAuth1 = Outside.service(this,"gus.web.authorization.setauth1");
    }



    public synchronized void register(String key, Object obj) throws Exception
    {
        if(key.equals("file")){initFile(obj);return;}
        if(key.equals("url")){initURL(obj);return;}
        if(key.equals("login")){login = (String) obj;return;}
        if(key.equals("password")){password = (String) obj;return;}
        if(key.equals("progress")){initProgress(obj);return;}
        if(key.equals("stop")){stop = true;return;}
        
        throw new Exception("Unknown key: "+key);
    }
    
    
    private void initFile(Object obj) throws Exception
    {
        if(obj instanceof File)
        {file = (File)obj;return;}
        if(obj instanceof String)
        {file = new File((String)obj);return;}
        throw new Exception("Invalid data type: "+obj.getClass().getName());
    }
    
    
    private void initURL(Object obj) throws Exception
    {
        if(obj instanceof URL)
        {url = (URL)obj;return;}
        if(obj instanceof File)
        {url = ((File)obj).toURI().toURL();return;}
        if(obj instanceof String)
        {url = new URL((String)obj);return;}
        throw new Exception("Invalid data type: "+obj.getClass().getName());
    }
    
    
    private void initProgress(Object obj)
    {
        progress_e = (Execute)obj;
        progress_reg = (Register)obj;
    }
    
    
    



    public void run()
    {
        if(progress_e==null) download();
        else download_p();
    }
    
    
    
    
    private void download()
    {
        try
        {
            stop = false;
            HttpURLConnection con = (HttpURLConnection)url.openConnection();
            if(login!=null && password!=null) setAuthorization(con);
            con.connect();
            
            FileOutputStream fos = new FileOutputStream(file);  
            InputStream is = con.getInputStream(); 
            
            downloadStarted();
            int b;
            while(((b = is.read())!=-1)&&(!stop))
            {
                fos.write(b);
                Thread.yield();
            }    
            is.close();
            fos.close();
            con.disconnect();
            
            if(stop) downloadAborted();
            else downloadCompleted();
        }
        catch(Exception e)
        {
            Outside.err(this,"download()",e);
            downloadFailed();
        }
    }
    
    
    
    
    
    
    
    
    
    private void download_p()
    {
        try
        {
            stop = false;
            HttpURLConnection con = (HttpURLConnection) url.openConnection();
            if(login!=null && password!=null) setAuthorization(con);
            con.connect();
            
            FileOutputStream fos = new FileOutputStream(file);  
            InputStream is = con.getInputStream(); 
            
            progress_reg.register("size",""+con.getContentLength());
            
            downloadStarted();
            int b;
            while(((b = is.read())!=-1)&&(!stop)) 
            {
                fos.write(b);
                progress_e.execute();
                Thread.yield();
            }    
            is.close();
            fos.close();
            con.disconnect();
            
            if(stop) downloadAborted();
            else downloadCompleted();
        }
        catch(Exception e)
        {
            Outside.err(this,"download_p()",e);
            downloadFailed();
        }
    }


    
    
    
    private void setAuthorization(HttpURLConnection con) throws Exception
    {setAuth1.give(new Object[]{con,login,password});}
    
    
    
    
    private void downloadStarted()
    {send(this,"downloadStarted()");}
    
    private void downloadAborted()
    {send(this,"downloadAborted()");}
    
    private void downloadCompleted()
    {send(this,"downloadCompleted()");}
    
    private void downloadFailed()
    {send(this,"downloadFailed()");}
    
}

