package gus05.manager.gus.endorf.layer2;

import gus05.framework.core.Entity;
import gus05.framework.features.Execute;
import gus05.framework.features.Register;
import gus05.framework.tools.DefaultSupport;
import gus05.manager.gus.endorf.layer1.Endorf;
import gus05.manager.gus.endorf.layer1.EndorfManager;
import gus05.manager.gus.endorf.layer1.Source;
import gus05.manager.gus.endorf.layer1.organs.ResourceProvider;
import gus05.manager.gus.endorf.layer2.datasupport.SupportList;
import gus05.manager.gus.endorf.layer2.datasupport.SupportMap;
import gus05.manager.gus.endorf.layer2.errorsupport.ErrorSupport;
import gus05.manager.gus.endorf.layer2.featuresupport.FeatureSupport;
import gus05.manager.gus.endorf.layer2.printstream.PrintStream1;
import gus05.manager.gus.endorf.layer2.printstream.PrintStreamErr;
import gus05.manager.gus.endorf.layer2.printstream.PrintStreamOut;
import gus05.manager.gus.endorf.layer3.system1.Class0;
import gus05.manager.gus.endorf.layer3.system1.MapClass0;

import java.io.File;
import java.io.FileFilter;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.swing.JOptionPane;


public class EndorfSystem {

    public static final String PARAM_APPARGS                    = "app.args";
    public static final String PARAM_APPOBJECT                  = "app.object";
    public static final String PARAM_APPSTARTDATE               = "app.startdate";
    public static final String PARAM_APPNAME                    = "app.name";
    
    public static final String INTERNAL_MAP_INTERNAL            = "app.internalmap";
    public static final String INTERNAL_MAP_PROVIDERS           = "app.providersmap";
    public static final String INTERNAL_MAP_CLASSENTITY         = "app.entityclassmap";
    public static final String INTERNAL_MAP_PROPERTIES          = "app.prop.map";
    public static final String INTERNAL_LIST_START              = "app.start.list";
    public static final String INTERNAL_MAP_CALLMAPPING         = "app.mapping";
    public static final String INTERNAL_OBJ_MAPPINGRECORDER     = "app.mappingrecorder";
    
    public static final String INTERNAL_PRINTSTREAM_OUT         = "app.out";
    public static final String INTERNAL_SUPPORT_ERROR           = "app.supporterror";
    public static final String INTERNAL_SUPPORT_FEATURE         = "app.supportfeature";
    public static final String INTERNAL_SUPPORT_START           = "app.supportstart";
    public static final String INTERNAL_SUPPORT_EXIT            = "app.supportexit";
    public static final String INTERNAL_EXECUTE_BEFOREXIT       = "app.executebeforeexit";
    
    public static final String PROPKEY_PRINTSTACKTRACE          = "app.debug.printstacktrace";
    public static final String PROPKEY_SYSTEMOUT                = "app.debug.systemout";
    
    public static final String RESOURCEPROVIDER_INTERNAL        = "internal";
    public static final String RESOURCEPROVIDER_ENTITY          = "entity";
    
    public static final String FEATURENAME_EXECUTE              = "execute";
    public static final String FEATURENAME_FILTER               = "filter";
    public static final String FEATURENAME_FUNCTION             = "function";
    public static final String FEATURENAME_GIVE                 = "give";
    public static final String FEATURENAME_PROVIDE              = "provide";
    public static final String FEATURENAME_REGISTER             = "register";
    public static final String FEATURENAME_RETRIEVE             = "retrieve";
    public static final String FEATURENAME_TRANSFORM            = "transform";
    public static final String FEATURENAME_GRAPHIC              = "graphic";
    public static final String FEATURENAME_SUPPORT_ADD          = "support_add";
    public static final String FEATURENAME_SUPPORT_REMOVE       = "support_remove";
    public static final String FEATURENAME_SUPPORT_LIST         = "support_list";
    public static final String FEATURENAME_RUN                  = "run";
    
    
    
    
    
    
    
    
    
    
    
    public static Date appStartDate;
    public static Object appObject;
    public static String[] appArgs;
    public static String appName;
    
    public static Map internalMap;
    public static Map providerMap;
    public static MapClass0 entityMap;
    public static Map mapping;
    public static Map properties;
    public static List startList;
    
    public static PrintStream1 appOut;
    public static PrintStreamErr err;
    public static PrintStreamOut out;
    
    public static ErrorSupport errorSupport;
    public static FeatureSupport featureSupport;
    
    public static DefaultSupport startSupport;
    public static DefaultSupport exitSupport;
    public static DefaultSupport finalTasks;
    
    public static boolean debug = false;
    public static boolean running = false;
    
    
    public static FileFilter jarFilter = new FileFilter(){
        public boolean accept(File f) {return f.getName().toLowerCase().endsWith(".jar");}
    };
    
    
    
    
    
    
    public static void init(Object object, Date startDate, String name, String[] args)
    {
        appStartDate = startDate;
        appObject = object;
        appName = name;
        formatArgs(args);
        
        internalMap = new SupportMap("internalMap");
        providerMap = new SupportMap("providerMap");
        entityMap = new MapClass0("entityMap");
        mapping = new SupportMap("mapping");
        properties = new SupportMap("properties");
        startList = new SupportList("startList");
        
        errorSupport = new ErrorSupport();
        featureSupport = new FeatureSupport();
        
        appOut = new PrintStream1();
        err = new PrintStreamErr(appOut);
        out = new PrintStreamOut(appOut);
        System.setErr(err);
        System.setOut(out);
        
        startSupport = new DefaultSupport();
        exitSupport = new DefaultSupport();
        finalTasks = new DefaultSupport();
        
        
        debug = hasArg("debug");
        
        if(debug)
        {
            appOut.println("DEBUG MODE ACTIVATED for application "+appName);
            appOut.println();
            printDebugInfo_Java();
            appOut.println();
            printDebugInfo_App(args);
            appOut.println();
            appOut.println();
        }
        
        
        
        
        putInternal(INTERNAL_MAP_INTERNAL,internalMap);
        putInternal(INTERNAL_MAP_PROVIDERS,providerMap);
        putInternal(INTERNAL_MAP_CLASSENTITY,entityMap);
        putInternal(INTERNAL_MAP_CALLMAPPING,mapping);
        putInternal(INTERNAL_MAP_PROPERTIES,properties);
        putInternal(INTERNAL_LIST_START,startList);
        putInternal(INTERNAL_PRINTSTREAM_OUT,appOut);
        putInternal(INTERNAL_SUPPORT_ERROR,errorSupport);
        putInternal(INTERNAL_SUPPORT_FEATURE,featureSupport);
        putInternal(INTERNAL_SUPPORT_START,startSupport);
        putInternal(INTERNAL_SUPPORT_EXIT,exitSupport);
        
        putInternal(PARAM_APPOBJECT,object);
        putInternal(PARAM_APPSTARTDATE,startDate);
        putInternal(PARAM_APPNAME,name);
        putInternal(PARAM_APPARGS,args);
    }
    
    
    
    
    
    
    
    
    
    private static void printDebugInfo_Java()
    {
        String runtimeVersion = System.getProperty("java.runtime.version");
        String homeDir = System.getProperty("java.home");
        String osname = System.getProperty("os.name");
        String osversion = System.getProperty("os.version");
        String[] paths = System.getProperty("java.class.path").split(File.pathSeparator);
        String[] ext = System.getProperty("java.ext.dirs").split(File.pathSeparator);
        
        appOut.println(" Java Runtime Version: "+runtimeVersion);
        appOut.println(" Java Home Dir: "+homeDir);
        appOut.println(" OS: "+osname+" "+osversion);
        appOut.println();
        appOut.println(" Java class path: "+paths.length+" path(s)");
        for(int i=0;i<paths.length;i++) appOut.println("  -path"+i+"="+paths[i]);
        appOut.println();
        appOut.println(" Java lib ext dirs: "+ext.length+" dir(s)");
        for(int i=0;i<ext.length;i++)
        {
            File extDir = new File(ext[i]);
            appOut.println("  -dir"+i+": "+extDir.getAbsolutePath());
            if(!extDir.exists()) appOut.println("   -does not exist !!");
            else
            {
                File[] jars = extDir.listFiles(jarFilter);
                for(int j=0;j<jars.length;j++) appOut.println("   -jar"+j+": "+jars[j].getName());
            }
        }
    }
    
    
    
    
    private static void printDebugInfo_App(String[] args)
    {
        appOut.println(" App name: "+appName);
        appOut.println(" App start date: "+yyyyMMdd_HHmmss.format(appStartDate));
        appOut.println(" App manager class: "+EndorfManager.class.getName());
        appOut.println(" App system class: "+EndorfSystem.class.getName());
        appOut.println(" App main class: "+appObject.getClass().getName());
        appOut.println();
        appOut.println(" App args: "+args.length+" argument(s)");
        for(int i=0;i<args.length;i++) appOut.println("  -arg"+i+"="+args[i]);
        appOut.println();
        appOut.println(" App formatted args: "+appArgs.length+" argument(s)");
        for(int i=0;i<appArgs.length;i++) appOut.println("  -arg"+i+"="+appArgs[i]);
    }
    
    
    
    
    
    
    
    
    public static void start()
    {
        running = true;
        d("start()", "dispatching startSupport");
        startSupport.send(EndorfSystem.class,"start()");
    }
    
    public static void exit()
    {
        m("exit()", "dispatching exitSupport ("+exitSupport.listeners().size()+" tasks)");
        exitSupport.send(EndorfSystem.class,"exit()");
        m("exit()", "dispatching finalTasks ("+finalTasks.listeners().size()+" tasks)");
        finalTasks.send(EndorfSystem.class,"exit()");
        beforeExit();
        
        m("exit()", "run time: "+fromStart());
        m("exit()", "System.exit(0)");
        appOut.close();
        System.exit(0);
    }
    
    
    
    private static void beforeExit()
    {
    	try
        {
    		Execute execute = (Execute) getInternal(INTERNAL_EXECUTE_BEFOREXIT);
            if(execute==null)return;
            
            m("beforeExit()", "before exit");
            execute.execute();
        }
        catch(Exception e)
        {e("recordMapping(String,String)",e);}
    }
    
    
    
    
    
    
    
    //*******************************************************
    // Internal
    
    public static boolean isInternal(String key)
    {return internalMap.containsKey(key);}
    
    public static void putInternal(String key, Object obj)
    {internalMap.put(key,obj);}
    
    
    public static Object getInternal(String key)
    {
        if(mapping.containsKey(key))
        {
            try{return Endorf.provideResource(null,(String) mapping.get(key));}
            catch(Exception e){e("getInternal(String)",new Exception(key+": "+e));}
        }
        Object internal = internalMap.get(key);
        return internal;
    }
    
    

    
    
    //*******************************************************
    // Mapping
    
    public static void recordMapping(String key, String value)
    {
        try
        {
            Register recorder = (Register) getInternal(INTERNAL_OBJ_MAPPINGRECORDER);
            if(recorder==null)return;
            recorder.register(key,value);
        }
        catch(Exception e)
        {e("recordMapping(String,String)",e);}
    }
    
    
    
    
    
    
    
    
    
    
    //*******************************************************
    // Resource providers
    
    public static boolean isProvider(String key)
    {return providerMap.containsKey(key);}

    public static void putProvider(String key, ResourceProvider provider)
    {providerMap.put(key,provider);}
    
    public static ResourceProvider getProvider(String key)
    {return (ResourceProvider) providerMap.get(key);}
    
    

    
    
    //*******************************************************
    // Entity classes

    public static boolean isEntity(String key)
    {return entityMap.containsKey(key);}
    
    public static void putEntityClass0(String key, Class0 c)
    {entityMap.put(key,c);}
    
    public static Class getEntityClass(String key) throws Exception
    {return (Class) entityMap.get(key);}
    
    
    
    
    
    
    //*******************************************************
    // Properties
    
    public static boolean hasProperty(String key)
    {return properties.containsKey(key);}
    
    public static String getProperty(String key)
    {return (String) properties.get(key);}
    
    public static String getProperty(String key, String defaultValue)
    {return hasProperty(key)? getProperty(key):defaultValue;}
    
    public static void setProperty(String name, String value)
    {properties.put(name,value);}
    
    public static boolean booleanProperty_true(String key)
    {
        if(!hasProperty(key))return true;
        return getProperty(key).toLowerCase().equals("true");
    }
    
    public static boolean booleanProperty_false(String key)
    {
        if(!hasProperty(key))return false;
        return getProperty(key).toLowerCase().equals("true");
    }
    
    
    
    public static boolean printStackTrace()
    {return debug || booleanProperty_true(PROPKEY_PRINTSTACKTRACE);}
    
    public static boolean systemOutEnabled()
    {return debug || booleanProperty_true(PROPKEY_SYSTEMOUT);}
    
    
    
    
    
    
    
    
    //*******************************************************
    // Feature support
    
    public static void featureStart(Source source, Object target, String feature, Object input)
    {featureSupport.featureStart(source, target, feature, input);}
    
    public static void featureEnd(Source source, Object target, String feature, Object output)
    {featureSupport.featureEnd(source, target, feature, output);}
    
    
    
    
    
    
    
    //*******************************************************
    // Time
    
    public static SimpleDateFormat yyyyMMdd_HHmmss = new SimpleDateFormat("yyyyMMdd_HHmmss");
    
    public static String now(){return yyyyMMdd_HHmmss.format(new Date());}
    
    public static long fromStart(){return System.currentTimeMillis()-appStartDate.getTime();}
    
    
    
    
    
    //*******************************************************
    // Args
    
    private static void formatArgs(String[] params)
    {
        StringBuilder b = new StringBuilder();
        for(int i=0;i<params.length;i++) b.append(params[i]+" ");
        if(b.length()>0) b.deleteCharAt(b.length()-1);
        appArgs = b.toString().split(";",-1);
    }
    
    public static boolean hasArg(String name)
    {
        name = name.toLowerCase();
        for(int i=0;i<appArgs.length;i++)
            if(appArgs[i].toLowerCase().equals(name)) return true;
        return false;
    }
    
    
    
    
    
    
    
    //*******************************************************
    // Messages
    
    public static void printInfo(String type, Object source, String id, String message)
    {appOut.println(now()+" "+phase+" "+errorNumber+" ["+type+"] "+sourceName(source)+"@"+id+": "+message);}
    
    public static void printMessage(Object source, String id, String message)
    {printInfo("M",source,id,message);}
    
    public static void printWarning(Object source, String id, String message)
    {printInfo("W",source,id,message);}
    
    public static void printDebug(Object source, String id, String message)
    {if(debug)printInfo("D",source,id,message);}
    
    
    
    
    public static int phase = 0;
    public static long phaseInstant = -1;
    public static NumberFormat nf = NumberFormat.getInstance();
    
    
    public static void changePhase(int newPhase)
    {
    	long previous = phaseInstant;
    	phaseInstant = System.nanoTime();
    	
    	if(previous!=-1)
    	appOut.println("-- phase "+phase+" duration: "+nf.format(phaseInstant-previous)+" ns");
        phase = newPhase;
        appOut.println("---------------------phase "+phase+"---------------------");
    }
    
    
    
    
    
    //*******************************************************
    // Errors
    
    private static int errorNumber = 0;
    public static int getErrorNumber() {return errorNumber;}
    
    
    public static void handleError(Object source, String id, Exception e)
    {
    	Entity entity = toEntity(source);
    	errorSupport.handleError(entity,id,e);
    	
        errorNumber++;
        printInfo("E",source,id,displayException(e));
        if(printStackTrace()){appOut.println();e.printStackTrace(appOut);}
    }
    
    
    private static String displayException(Exception e)
    {
        if(e.toString().startsWith("java.lang.Exception"))
            return e.getMessage();
        return e.toString();
    }
    
    
    public static String sourceName(Object source)
    {
        if(source==null) return "null";
        if(source instanceof Source) return source.toString();
        if(source instanceof Entity) return ((Entity)source).getName();
        if(source instanceof Class) return ((Class)source).getSimpleName();
        return source.getClass().getSimpleName();
    }
    
    
    public static Entity toEntity(Object obj)
    {
    	if(obj instanceof Entity) return (Entity) obj;
    	return new Entity0(obj);
    }
    
    
    private static class Entity0 implements Entity
    {
    	private Object obj;
    	public Entity0(Object obj) {this.obj=obj;}
		public String getCreationDate() {return "****.**.**";}
		public String getName() {return obj.getClass().getSimpleName();}
    }
    
    
    
    
    
    
    
    public static void FATAL_ERROR(Object source, String id, String message)
    {
        printInfo("X",source,id,message);
        String text = sourceName(source)+"@"+id+"\n"+message;
        JOptionPane.showMessageDialog(null,text,"FATAL ERROR",JOptionPane.ERROR_MESSAGE);
        
        if(!systemOutEnabled())
            appOut.initPrintStream(System.out);
        System.exit(1);
    }
    
    
    
    
    
    //********************************************
    // printing
    
    private static void m(String id, String message)
    {printMessage(EndorfSystem.class,id,message);}
    
    private static void w(String id, String message)
    {printWarning(EndorfSystem.class,id,message);}
    
    private static void d(String id, String message)
    {printDebug(EndorfSystem.class,id,message);}
    
    private static void e(String id, Exception e)
    {handleError(EndorfSystem.class,id,e);}
}
