package gus05.manager.gus.endorf.layer2.datasupport;

import java.awt.event.ActionListener;
import java.util.HashSet;
import java.util.List;


import javax.swing.SwingUtilities;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import gus05.framework.features.Register;
import gus05.framework.features.Retrieve;
import gus05.framework.features.Support;
import gus05.framework.tools.DefaultSupport;


public class SupportTree extends DefaultTreeModel implements Support, Register, Retrieve, CellEditorListener {

    
    
    public final static String DELIM = "\t";
    
    private DefaultSupport support;
    private String name;
    
    private TreePath before = null;
    private TreePath after = null;

    private DefaultMutableTreeNode currentNode;
    private int currentLevel;
    
    
    public String toString()
    {return name;}
    
    
    private void send(String id)
    {support.send(this,id);}
    
    
    
    public SupportTree(String name) throws Exception
    {this(name,"");}
    
    
    
    public SupportTree(String name, String value) throws Exception
    { 
        super(null);
        this.name = name;
        support = new DefaultSupport();
        
        currentNode = null;
        currentLevel = -1;

        String[] lines = value.split("[\n\r]+");
        for(int i=0;i<lines.length;i++)
        {
            String line = lines[i];
            if(!line.equals("")) handleLine(i,line);
        }
        if(getRoot()==null)
            setRoot(new DefaultMutableTreeNode0("<>"));
    }
    
    
    
    
    
    
    private void handleLine(int index, String line) throws Exception
    {
        String[] tab = line.split(DELIM+"+",2);
        String tagName = "";
        if(tab.length==1)tagName = tab[0];
        if(tab.length>1)tagName = tab[1];
        int level = line.length()-tagName.length();
        
        if(level>0 || index==0)
        {
            // build next node
            if(level>currentLevel+1)
                throw new Exception("next node level can not be higher than "+currentLevel+1);
            
            DefaultMutableTreeNode0 newNode = new DefaultMutableTreeNode0(tagName);
            
            if(level<currentLevel+1)
            {
                int up = currentLevel-level+1;
                for(int j=0;j<up;j++)
                    currentNode = (DefaultMutableTreeNode0)currentNode.getParent();
            }
            
            if(currentNode==null)
                setRoot(newNode);
            else currentNode.add(newNode);
            nodeStructureChanged(currentNode);
            
            currentNode = newNode;
            currentLevel = level;
        }
    }
    
    
    
   
    
    
    
    
//    private DefaultMutableTreeNode0 returnChildWithName(DefaultMutableTreeNode0 parent, String name)
//    {
//        for(int i=0;i<parent.getChildCount();i++)
//        {
//            DefaultMutableTreeNode0 child = (DefaultMutableTreeNode0)parent.getChildAt(i);
//            if(child.toString().equals(name)) return child;
//        }
//        return null;
//    }
    
    
    
    
    
    
    
    
    private void childUp(DefaultMutableTreeNode0 child)
    {
        before = new TreePath(child.getPath());
        after = null;
        
        DefaultMutableTreeNode0 parent = (DefaultMutableTreeNode0)child.getParent();
        if(parent==null)return;
        int index = getIndexOfChild(parent,child);
        if(index==0)
        {
            DefaultMutableTreeNode0 previous = (DefaultMutableTreeNode0)parent.getPreviousSibling();
            if(previous==null)return;
            
            removeNodeFromParent(child);
            insertNodeInto(child,previous,previous.getChildCount());
            after = new TreePath(child.getPath());
        }
        else
        {
            removeNodeFromParent(child);
            insertNodeInto(child,parent,index-1);
            after = new TreePath(child.getPath());
        }
        send("childUp(DefaultMutableTreeNode0)");
    }
    
    
    
    
    
    private void childDown(DefaultMutableTreeNode0 child)
    {
        before = new TreePath(child.getPath());
        after = null;
        
        DefaultMutableTreeNode0 parent = (DefaultMutableTreeNode0)child.getParent();
        if(parent==null)return;
        int index = getIndexOfChild(parent,child);
        if(index==parent.getChildCount()-1)
        {
            DefaultMutableTreeNode0 next = (DefaultMutableTreeNode0)parent.getNextSibling();
            if(next==null)return;
            
            removeNodeFromParent(child);
            insertNodeInto(child,next,0);
            after = new TreePath(child.getPath());
        }
        else
        {
            removeNodeFromParent(child);
            insertNodeInto(child,parent,index+1);
            after = new TreePath(child.getPath());
        }
        send("childDown(DefaultMutableTreeNode0)");
    }
    
    
    
    
    
    private void decreaseNode(DefaultMutableTreeNode0 node)
    {
        before = new TreePath(node.getPath());
        after = null;
        
        DefaultMutableTreeNode0 previous = (DefaultMutableTreeNode0) node.getPreviousSibling();
        if(previous==null)return;
        
        removeNodeFromParent(node);
        insertNodeInto(node,previous,previous.getChildCount());
        after = new TreePath(node.getPath());
        
        send("decreaseNode(DefaultMutableTreeNode0)");
    }
    
    
    
    
    private void increaseNode(DefaultMutableTreeNode0 node)
    {
        before = new TreePath(node.getPath());
        after = null;
        
        DefaultMutableTreeNode0 parent = (DefaultMutableTreeNode0)node.getParent();
        if(parent==null)return;
        DefaultMutableTreeNode0 parent2 = (DefaultMutableTreeNode0)parent.getParent();
        if(parent2==null)return;
        
        int index = getIndexOfChild(parent2,parent)+1;
        removeNodeFromParent(node);
        insertNodeInto(node,parent2,index);
        after = new TreePath(node.getPath());
        
        send("increaseNode(DefaultMutableTreeNode0)");
    }
    
    
    
    
    private void deleteNode(DefaultMutableTreeNode0 node)
    {
        before = new TreePath(node.getPath());
        after = null;
        
        DefaultMutableTreeNode0 nextNode = (DefaultMutableTreeNode0) node.getPreviousSibling();
        if(nextNode==null) nextNode = (DefaultMutableTreeNode0) node.getNextSibling();
        if(nextNode==null) nextNode = (DefaultMutableTreeNode0) node.getParent();
        if(nextNode==null) return;
        
        removeNodeFromParent(node);
        after = new TreePath(nextNode.getPath());
        
        send("deleteNode(DefaultMutableTreeNode0)");
    }
    
    
    
    
    
    
    private void createNode(DefaultMutableTreeNode0 node)
    {
        before = null;
        
        DefaultMutableTreeNode0 parent = (DefaultMutableTreeNode0)node.getParent();
        if(parent==null)
        {
            DefaultMutableTreeNode0 newNode = new DefaultMutableTreeNode0("");
            insertNodeInto(newNode,node,0);
            after = new TreePath(newNode.getPath());
        }
        else
        {
            DefaultMutableTreeNode0 newNode = new DefaultMutableTreeNode0("");
            int index = getIndexOfChild(parent,node)+1;
            insertNodeInto(newNode,parent,index);
            after = new TreePath(newNode.getPath());
        }
        send("createNode(DefaultMutableTreeNode0)");
    }
    
    
    
    
    
    
    private void createSibling(DefaultMutableTreeNode0 node)
    {
        before = null;
        
        DefaultMutableTreeNode0 parent = (DefaultMutableTreeNode0)node.getParent();
        if(parent==null) return;

        DefaultMutableTreeNode0 newNode = new DefaultMutableTreeNode0("");
        int index = getIndexOfChild(parent,node)+1;
        insertNodeInto(newNode,parent,index);
        after = new TreePath(newNode.getPath());

        send("createSibling(DefaultMutableTreeNode0)");
    }    

    
    
    
    
    
    
    private void createChild(DefaultMutableTreeNode0 node)
    {
        before = null;
        DefaultMutableTreeNode0 newNode = new DefaultMutableTreeNode0("");
        insertNodeInto(newNode,node,0);
        after = new TreePath(newNode.getPath());
        send("createChild(DefaultMutableTreeNode0)");
    }
    
    
    
    
    
    private void addChild(DefaultMutableTreeNode0 parent, DefaultMutableTreeNode0 child)
    {
        before = null;
        after = new TreePath(child.getPath());
        insertNodeInto(child,parent,0);
        send("addChild(DefaultMutableTreeNode0,DefaultMutableTreeNode0)");
    }

    
    
    
    private void updateNode(DefaultMutableTreeNode0 node)
    {nodeChanged(node);}
    
    
    
    
    
    
    public void editingCanceled(ChangeEvent e){}



    public void editingStopped(ChangeEvent e)
    {
        SwingUtilities.invokeLater(new Runnable(){
            public void run() {send("editingStopped(ChangeEvent)");}
        });
    }
    
    
    

    public void register(String key, Object obj) throws Exception
    {
        if(key.equals("childUp")) childUp((DefaultMutableTreeNode0)obj);
        if(key.equals("childDown")) childDown((DefaultMutableTreeNode0)obj);
        if(key.equals("decreaseNode")) decreaseNode((DefaultMutableTreeNode0)obj);
        if(key.equals("increaseNode")) increaseNode((DefaultMutableTreeNode0)obj);
        if(key.equals("deleteNode")) deleteNode((DefaultMutableTreeNode0)obj);
        if(key.equals("createNode")) createNode((DefaultMutableTreeNode0)obj);
        if(key.equals("createChild")) createChild((DefaultMutableTreeNode0)obj);
        if(key.equals("createSibling")) createSibling((DefaultMutableTreeNode0)obj);
        if(key.equals("updateNode")) updateNode((DefaultMutableTreeNode0)obj);
        
        if(key.equals("addChild"))
        {
            DefaultMutableTreeNode0[] nodes = (DefaultMutableTreeNode0[])obj;
            addChild(nodes[0],nodes[1]);
        }
    }

    
    public Object retrieve(String key) throws Exception
    {
        if(key.equals("after")) return after;
        if(key.equals("before")) return before;
        if(key.equals("description")) return description();
        
        if(key.equals("keys"))
            return new String[]{"after","before","description"};
        throw new Exception("Unknown key: "+key);
    }
    
    
    
    
    
//    private HashSet getAllNodes()
//    {
//        HashSet set = new HashSet();
//        addIntoSet(set,getRoot());
//        return set;
//    }
    
    private void addIntoSet(HashSet set, Object node)
    {
        set.add(node.toString());
        for(int i=0;i<getChildCount(node);i++)
        addIntoSet(set,getChild(node,i));
    }
        
    
    
    
    
    public String description()
    {
        StringBuffer b = new StringBuffer();
        buildRepresentation(b,getRoot(),0);
        return b.toString();
    }
    
    private void buildRepresentation(StringBuffer b, Object node, int level)
    {
        for(int i=0;i<level;i++) b.append(DELIM);
        b.append(node.toString()+"\n");
        for(int i=0;i<getChildCount(node);i++)
            buildRepresentation(b,getChild(node,i),level+1);
    }


    
    
    
    
    


    public void addActionListener(ActionListener listener)
    {support.addActionListener(listener);}
    
    public void removeActionListener(ActionListener listener)
    {support.removeActionListener(listener);}
    
    public List listeners()
    {return support.listeners();}
    
    
    
    
    
    
    
    
    
    private class DefaultMutableTreeNode0 extends DefaultMutableTreeNode
    {
        public DefaultMutableTreeNode0(Object userObject)
        {super(userObject);}
        
        public void setUserObject(Object userObject)
        {
            super.setUserObject(userObject);
            send("setUserObject(Object)");
        }
    }

    
}
