/*
 * Created on 21 juil. 2006
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package gus05.manager.gus.endorf.layer4.internals1;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.util.Properties;

import gus05.framework.features.Transform;
import gus05.manager.gus.endorf.layer3.tools.SECURITY;
import gus05.manager.gus.endorf.layer3.tools.Tools_dialog;
import gus05.manager.gus.endorf.layer3.tools.Tools_file;
import gus05.manager.gus.endorf.layer3.tools.Tools_fileFilter;
import gus05.manager.gus.endorf.layer3.tools.Tools_publicKey;
import gus05.manager.gus.endorf.layer4.system2.System2;



public class AppPrivateKeySigner implements Transform {

    
    
    private File securityDir;
    private File privateKeyFile;
    private File publicKeyFile;
    
    private String userName;
    private byte[] userSign;
    private PrivateKey privateKey;
    private PublicKey publicKey;
    
    private KeyPairGenerator kpg;
    private Signature si;
    
    
    
    
    public PublicKey getPublicKey()
    {return publicKey;}
    
    
    
    
    
    
    
    
    
    public AppPrivateKeySigner()
    {init();}
    
    

    public void init()
    {
        try
        {
            securityDir = System2.getFile(System2.PATH_SECURITYDIR);
            if(!securityDir.exists()) securityDir.mkdirs();
            
            si = Signature.getInstance(SECURITY.SECURITY_ALGO_DSA);
            kpg = KeyPairGenerator.getInstance(SECURITY.SECURITY_ALGO_DSA);
            kpg.initialize(512);
            
            File[] f = securityDir.listFiles(Tools_fileFilter.privatekeyFileFilter);
            if(f.length>1) manyPrivateKeysDetected();
            if(f.length==0) createNewID();
            else loadUserID(f[0]);
            
            checkReadOnly(privateKeyFile);
            checkReadOnly(publicKeyFile);
            
            System2.pKeyMap.put(userName, publicKey);
        }
        catch(Exception e)
        {System2.handleError(this,"init()",e);}
    }
    
    
    
    
    
    private void manyPrivateKeysDetected()
    {
        String message = "many private keys have been detected in security directory "+securityDir.getAbsolutePath();
        System2.FATAL_ERROR(this,"manyPrivateKeysDetected()",message);
    }
    
    
    
    private void createNewID() throws Exception
    {
        userName = Tools_dialog.input("User identification","Please type your personal identification:");
        if(userName==null) userName = "UNDEFINED";
        System2.setProperty(System2.PROPKEY_APPUSER,userName);
        
        privateKeyFile = new File(securityDir,userName+".privatekey");
        publicKeyFile = new File(securityDir,userName+".publickey");
        
        generateNewKeys();
        generateUserSign();
        storePrivateKey();
        storePublicKey();
        
        Tools_dialog.warning("ID created","Your personal private key and public key files" +
        		" have been created inside security dir:\n" +securityDir.getAbsolutePath());
        
        System2.printMessage(this,"createNewID()","app user: "+userName);
    }
    
    
    
    
    private void loadUserID(File file) throws Exception
    {
        userName = getFileName(file);
        System2.setProperty(System2.PROPKEY_APPUSER,userName);
        
        privateKeyFile = file;
        publicKeyFile = new File(securityDir,userName+".publickey");
        
        loadPrivateKey();
        loadPublicKey();
        generateUserSign();
        
        System2.printMessage(this,"loadUserID(File)","app user: "+userName);
    }
    
    
    
    
    
    
    
    
    
    
    
    
    
    private void generateNewKeys() throws Exception
    {
        KeyPair kp = kpg.generateKeyPair();
        privateKey = kp.getPrivate();
        publicKey = kp.getPublic();
    }
    
    
    private void generateUserSign() throws Exception
    {
    	si.initSign(privateKey);
        si.update(userName.getBytes());
        userSign = si.sign();
    }

    
    
    private void loadPrivateKey() throws Exception
    {
        Properties prop = Tools_file.prop(privateKeyFile);
        String owner = prop.getProperty("owner");
        if(!owner.equals(userName)) throw new Exception("Invalid owner in file "+privateKeyFile.getName());
        privateKey = Tools_publicKey.buildPrivateKey(prop);
    } 
    
    
    
    private void loadPublicKey() throws Exception
    {
        Properties prop = Tools_file.prop(publicKeyFile);
        String owner = prop.getProperty("owner");
        if(!owner.equals(userName)) throw new Exception("Invalid owner in file "+publicKeyFile.getName());
        publicKey = Tools_publicKey.buildPublicKey(prop);
    }

    
    
    
    
    
    
    
    
    
    
    
    
    private void storePrivateKey() throws Exception
    {
        Properties prop = Tools_publicKey.privateProp(privateKey,userName,userSign);
        prop.store(new FileOutputStream(privateKeyFile),"private key specifications for "+userName);
    }
    
    
    private void storePublicKey() throws Exception
    {
        Properties prop = Tools_publicKey.publicProp(publicKey,userName,userSign);
        prop.store(new FileOutputStream(publicKeyFile), "public key specifications for "+userName); 
    }

    
    
    
    
    
    
    
    
    
    
    /*
     * Attention, sur cette mthode le synchronized est trs important
     * La cration de la signature prend du temps et est sensible au multithreading
     */
    public synchronized Object transform(Object obj) throws Exception
    {
        if(si==null)
            throw new Exception("Private key signer is not available");
        if(obj instanceof byte[])
        {
            byte[] data = (byte[])obj;
            si.update(data);
        }
        if(obj instanceof String)
        {
            String data = (String)obj;
            si.update(data.getBytes());
        }
        if(obj instanceof InputStream)
        {
            InputStream in = (InputStream)obj;
            byte[] buffer = new byte[8192];
            int length;
            while((length=in.read(buffer))!=-1)
            si.update(buffer, 0, length);
        }
        return si.sign();
    }
    
    

    
    
    private void checkReadOnly(File file)
    {if(file.canWrite()) file.setReadOnly();}
    
    
    
    private String getFileName(File file)
    {
        String fullName = file.getName();
        if(!fullName.contains(".")) return fullName;
        
        String[] n = fullName.split("\\.");
        int l = n[n.length-1].length();
        return fullName.substring(0,fullName.length()-1-l);
    }
}
